@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2013,2014,2016 Olly Betts
  * Copyright 2006,2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -33,15 +33,17 @@
 
 #include "omassert.h"
 #include "debuglog.h"
-#include "../backends/multi/multi_postlist.h"
-#include "../backends/multi/multi_termlist.h"
-#include "alltermslist.h"
-#include "multialltermslist.h"
-#include "multivaluelist.h"
-#include "database.h"
+#include "backends/alltermslist.h"
+#include "backends/multi/multi_alltermslist.h"
+#include "backends/multi/multi_postlist.h"
+#include "backends/multi/multi_termlist.h"
+#include "backends/multivaluelist.h"
+#include "backends/database.h"
 #include "editdistance.h"
-#include "ortermlist.h"
+#include "expand/ortermlist.h"
+#include "internaltypes.h"
 #include "noreturn.h"
+#include "pack.h"
 
 #include <algorithm>
 #include <cstdlib> // For abs().
@@ -49,6 +51,7 @@
 #include <vector>
 
 using namespace std;
+using Xapian::Internal::intrusive_ptr;
 
 XAPIAN_NORETURN(static void docid_zero_invalid());
 static void docid_zero_invalid()
@@ -59,7 +62,25 @@ static void docid_zero_invalid()
 XAPIAN_NORETURN(static void no_subdatabases());
 static void no_subdatabases()
 {
-    throw Xapian::DocNotFoundError("No subdatabases");
+    throw Xapian::InvalidOperationError("No subdatabases");
+}
+
+XAPIAN_NORETURN(static void empty_metadata_key());
+static void empty_metadata_key()
+{
+    throw Xapian::InvalidArgumentError("Empty metadata keys are invalid");
+}
+
+inline size_t
+sub_db(Xapian::docid did, size_t n_dbs)
+{
+    return (did - 1) % n_dbs;
+}
+
+inline size_t
+sub_docid(Xapian::docid did, size_t n_dbs)
+{
+    return (did - 1) / n_dbs + 1;
 }
 
 namespace Xapian {
@@ -72,7 +93,7 @@ Database::Database()
 Database::Database(Database::Internal *internal_)
 {
     LOGCALL_CTOR(API, "Database", internal_);
-    Xapian::Internal::RefCntPtr<Database::Internal> newi(internal_);
+    intrusive_ptr<Database::Internal> newi(internal_);
     internal.push_back(newi);
 }
 
@@ -94,21 +115,24 @@ Database::~Database()
     LOGCALL_DTOR(API, "Database");
 }
 
-void
+bool
 Database::reopen()
 {
-    LOGCALL_VOID(API, "Database::reopen", NO_ARGS);
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::iterator i;
+    LOGCALL(API, bool, "Database::reopen", NO_ARGS);
+    bool maybe_changed = false;
+    vector<intrusive_ptr<Database::Internal> >::iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
-	(*i)->reopen();
+	if ((*i)->reopen())
+	    maybe_changed = true;
     }
+    RETURN(maybe_changed);
 }
 
 void
 Database::close()
 {
     LOGCALL_VOID(API, "Database::close", NO_ARGS);
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::iterator i;
+    vector<intrusive_ptr<Database::Internal> >::iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
 	(*i)->close();
     }
@@ -122,7 +146,7 @@ Database::add_database(const Database & database)
 	LOGLINE(API, "Database added to itself");
 	throw Xapian::InvalidArgumentError("Can't add a Database to itself");
     }
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     for (i = database.internal.begin(); i != database.internal.end(); ++i) {
 	internal.push_back(*i);
     }
@@ -141,12 +165,12 @@ Database::postlist_begin(const string &tname) const
     if (internal.size() == 1)
 	RETURN(PostingIterator(internal[0]->open_post_list(tname)));
 
-    if (rare(internal.size() == 0))
+    if (rare(internal.empty()))
 	RETURN(PostingIterator());
 
     vector<LeafPostList *> pls;
     try {
-	vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+	vector<intrusive_ptr<Database::Internal> >::const_iterator i;
 	for (i = internal.begin(); i != internal.end(); ++i) {
 	    pls.push_back((*i)->open_post_list(tname));
 	    pls.back()->next();
@@ -189,12 +213,6 @@ Database::termlist_begin(Xapian::docid did) const
     RETURN(TermIterator(tl));
 }
 
-TermIterator
-Database::allterms_begin() const
-{
-    return allterms_begin(string());
-}
-
 TermIterator
 Database::allterms_begin(const std::string & prefix) const
 {
@@ -215,7 +233,7 @@ Database::has_positions() const
 {
     LOGCALL(API, bool, "Database::has_positions", NO_ARGS);
     // If any sub-database has positions, the combined database does.
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
 	if ((*i)->has_positions()) RETURN(true);
     }
@@ -244,7 +262,7 @@ Database::get_doccount() const
 {
     LOGCALL(API, Xapian::doccount, "Database::get_doccount", NO_ARGS);
     Xapian::doccount docs = 0;
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
 	docs += (*i)->get_doccount();
     }
@@ -270,19 +288,18 @@ Database::get_avlength() const
 {
     LOGCALL(API, Xapian::doclength, "Database::get_avlength", NO_ARGS);
     Xapian::doccount docs = 0;
-    Xapian::doclength totlen = 0;
+    totlen_t totlen = 0;
 
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
-	Xapian::doccount db_doccount = (*i)->get_doccount();
-	docs += db_doccount;
-	totlen += (*i)->get_avlength() * db_doccount;
+	docs += (*i)->get_doccount();
+	totlen += (*i)->get_total_length();
     }
     LOGLINE(UNKNOWN, "get_avlength() = " << totlen << " / " << docs <<
 	    " (from " << internal.size() << " dbs)");
 
     if (docs == 0) RETURN(0.0);
-    RETURN(totlen / docs);
+    RETURN(totlen / double(docs));
 }
 
 Xapian::doccount
@@ -292,9 +309,11 @@ Database::get_termfreq(const string & tname) const
     if (tname.empty()) RETURN(get_doccount());
 
     Xapian::doccount tf = 0;
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
-	tf += (*i)->get_termfreq(tname);
+	Xapian::doccount sub_tf;
+	(*i)->get_freqs(tname, &sub_tf, NULL);
+	tf += sub_tf;
     }
     RETURN(tf);
 }
@@ -306,9 +325,11 @@ Database::get_collection_freq(const string & tname) const
     if (tname.empty()) RETURN(get_doccount());
 
     Xapian::termcount cf = 0;
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
-	cf += (*i)->get_collection_freq(tname);
+	Xapian::termcount sub_cf;
+	(*i)->get_freqs(tname, NULL, &sub_cf);
+	cf += sub_cf;
     }
     RETURN(cf);
 }
@@ -319,7 +340,7 @@ Database::get_value_freq(Xapian::valueno slot) const
     LOGCALL(API, Xapian::doccount, "Database::get_value_freq", slot);
 
     Xapian::doccount vf = 0;
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
 	vf += (*i)->get_value_freq(slot);
     }
@@ -333,7 +354,7 @@ Database::get_value_lower_bound(Xapian::valueno slot) const
 
     if (rare(internal.empty())) RETURN(string());
 
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     i = internal.begin();
     string full_lb = (*i)->get_value_lower_bound(slot);
     while (++i != internal.end()) {
@@ -349,7 +370,7 @@ Database::get_value_upper_bound(Xapian::valueno slot) const
     LOGCALL(API, std::string, "Database::get_value_upper_bound", slot);
 
     std::string full_ub;
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
 	std::string ub = (*i)->get_value_upper_bound(slot);
 	if (ub > full_ub)
@@ -366,7 +387,7 @@ Database::get_doclength_lower_bound() const
     if (rare(internal.empty())) RETURN(0);
 
     Xapian::termcount full_lb = 0;
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
 	// Skip sub-databases which are empty or only contain documents with
 	// doclen==0.
@@ -384,7 +405,7 @@ Database::get_doclength_upper_bound() const
     LOGCALL(API, Xapian::termcount, "Database::get_doclength_upper_bound", NO_ARGS);
 
     Xapian::termcount full_ub = 0;
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
 	Xapian::termcount ub = (*i)->get_doclength_upper_bound();
 	if (ub > full_ub) full_ub = ub;
@@ -399,7 +420,7 @@ Database::get_wdf_upper_bound(const string & term) const
     if (term.empty()) RETURN(0);
 
     Xapian::termcount full_ub = 0;
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
 	Xapian::termcount ub = (*i)->get_wdf_upper_bound(term);
 	if (ub > full_ub) full_ub = ub;
@@ -412,7 +433,7 @@ Database::valuestream_begin(Xapian::valueno slot) const
 {
     LOGCALL(API, ValueIterator, "Database::valuestream_begin", slot);
     if (internal.size() == 0)
-       	RETURN(ValueIterator());
+	RETURN(ValueIterator());
     if (internal.size() != 1)
 	RETURN(ValueIterator(new MultiValueList(internal, slot)));
     RETURN(ValueIterator(internal[0]->open_value_list(slot)));
@@ -433,6 +454,20 @@ Database::get_doclength(Xapian::docid did) const
     RETURN(internal[n]->get_doclength(m));
 }
 
+Xapian::termcount
+Database::get_unique_terms(Xapian::docid did) const
+{
+    LOGCALL(API, Xapian::termcount, "Database::get_unique_terms", did);
+    if (did == 0)
+	docid_zero_invalid();
+    unsigned int multiplier = internal.size();
+    if (rare(multiplier == 0))
+	no_subdatabases();
+    Xapian::doccount n = (did - 1) % multiplier; // which actual database
+    Xapian::docid m = (did - 1) / multiplier + 1; // real docid in that database
+    RETURN(internal[n]->get_unique_terms(m));
+}
+
 Document
 Database::get_document(Xapian::docid did) const
 {
@@ -450,21 +485,6 @@ Database::get_document(Xapian::docid did) const
     RETURN(Document(internal[n]->open_document(m, false)));
 }
 
-Document::Internal *
-Database::get_document_lazily(Xapian::docid did) const
-{
-    LOGCALL(DB, Document::Internal *, "Database::get_document_lazily", did);
-    if (did == 0)
-	docid_zero_invalid();
-
-    unsigned int multiplier = internal.size();
-    Assert(multiplier != 0);
-    Xapian::doccount n = (did - 1) % multiplier; // which actual database
-    Xapian::docid m = (did - 1) / multiplier + 1; // real docid in that database
-
-    RETURN(internal[n]->open_document(m, true));
-}
-
 bool
 Database::term_exists(const string & tname) const
 {
@@ -472,7 +492,7 @@ Database::term_exists(const string & tname) const
     if (tname.empty()) {
 	RETURN(get_doccount() != 0);
     }
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
 	if ((*i)->term_exists(tname)) RETURN(true);
     }
@@ -483,7 +503,7 @@ void
 Database::keep_alive()
 {
     LOGCALL_VOID(API, "Database::keep_alive", NO_ARGS);
-    vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
+    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
     for (i = internal.begin(); i != internal.end(); ++i) {
 	(*i)->keep_alive();
     }
@@ -492,7 +512,7 @@ Database::keep_alive()
 string
 Database::get_description() const
 {
-    /// \todo display contents of the database
+    /// @todo display contents of the database
     return "Database()";
 }
 
@@ -556,18 +576,9 @@ Database::get_spelling_suggestion(const string &word,
     if (!merger.get()) RETURN(string());
 
     // Convert word to UTF-32.
-#if ! defined __SUNPRO_CC || __SUNPRO_CC - 0 >= 0x580
     // Extra brackets needed to avoid this being misparsed as a function
     // prototype.
     vector<unsigned> utf32_word((Utf8Iterator(word)), Utf8Iterator());
-#else
-    // Older versions of Sun's C++ compiler need this workaround, but 5.8
-    // doesn't.  Unsure of the exact version it was fixed in.
-    vector<unsigned> utf32_word;
-    for (Utf8Iterator sunpro_it(word); sunpro_it != Utf8Iterator(); ++sunpro_it) {
-	utf32_word.push_back(*sunpro_it);
-    }
-#endif
 
     vector<unsigned> utf32_term;
 
@@ -690,7 +701,7 @@ Database::synonyms_begin(const std::string &term) const
 TermIterator
 Database::synonym_keys_begin(const std::string &prefix) const
 {
-    LOGCALL(API, TermIterator, "Database::synonyms_keys_begin", prefix);
+    LOGCALL(API, TermIterator, "Database::synonym_keys_begin", prefix);
     AutoPtr<TermList> merger;
     for (size_t i = 0; i < internal.size(); ++i) {
 	TermList * tl = internal[i]->open_synonym_keylist(prefix);
@@ -709,8 +720,8 @@ string
 Database::get_metadata(const string & key) const
 {
     LOGCALL(API, string, "Database::get_metadata", key);
-    if (key.empty())
-	throw InvalidArgumentError("Empty metadata keys are invalid");
+    if (rare(key.empty()))
+	empty_metadata_key();
     if (internal.empty()) RETURN(std::string());
     RETURN(internal[0]->get_metadata(key));
 }
@@ -740,6 +751,24 @@ Database::get_uuid() const
     RETURN(uuid);
 }
 
+Xapian::rev
+Database::get_revision() const
+{
+    LOGCALL(API, Xapian::rev, "Database::get_revision", NO_ARGS);
+    size_t n_dbs = internal.size();
+    if (rare(n_dbs != 1))
+	throw Xapian::InvalidOperationError("Database::get_revision() requires "
+					    "exactly one subdatabase");
+    const string& s = internal[0]->get_revision_info();
+    const char* p = s.data();
+    const char* end = p + s.size();
+    Xapian::rev revision;
+    if (!unpack_uint(&p, end, &revision))
+	throw Xapian::UnimplementedError("Database::get_revision() only "
+					 "supported for chert and glass");
+    return revision;
+}
+
 ///////////////////////////////////////////////////////////////////////////
 
 WritableDatabase::WritableDatabase() : Database()
@@ -771,42 +800,48 @@ WritableDatabase::~WritableDatabase()
     LOGCALL_DTOR(API, "WritableDatabase");
 }
 
-XAPIAN_NORETURN(static void only_one_subdatabase_allowed());
-static void only_one_subdatabase_allowed()
-{
-    throw Xapian::InvalidOperationError("WritableDatabase needs exactly one subdatabase");
-}
-
 void
 WritableDatabase::commit()
 {
     LOGCALL_VOID(API, "WritableDatabase::commit", NO_ARGS);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
-    internal[0]->commit();
+    size_t n_dbs = internal.size();
+    if (rare(n_dbs == 0))
+	no_subdatabases();
+    for (size_t i = 0; i != n_dbs; ++i)
+	internal[i]->commit();
 }
 
 void
 WritableDatabase::begin_transaction(bool flushed)
 {
-    LOGCALL_VOID(API, "WritableDatabase::begin_transaction", NO_ARGS);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
-    internal[0]->begin_transaction(flushed);
+    LOGCALL_VOID(API, "WritableDatabase::begin_transaction", flushed);
+    size_t n_dbs = internal.size();
+    if (rare(n_dbs == 0))
+	no_subdatabases();
+    for (size_t i = 0; i != n_dbs; ++i)
+	internal[i]->begin_transaction(flushed);
 }
 
 void
 WritableDatabase::commit_transaction()
 {
     LOGCALL_VOID(API, "WritableDatabase::commit_transaction", NO_ARGS);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
-    internal[0]->commit_transaction();
+    size_t n_dbs = internal.size();
+    if (rare(n_dbs == 0))
+	no_subdatabases();
+    for (size_t i = 0; i != n_dbs; ++i)
+	internal[i]->commit_transaction();
 }
 
 void
 WritableDatabase::cancel_transaction()
 {
     LOGCALL_VOID(API, "WritableDatabase::cancel_transaction", NO_ARGS);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
-    internal[0]->cancel_transaction();
+    size_t n_dbs = internal.size();
+    if (rare(n_dbs == 0))
+	no_subdatabases();
+    for (size_t i = 0; i != n_dbs; ++i)
+	internal[i]->cancel_transaction();
 }
 
 
@@ -814,38 +849,63 @@ Xapian::docid
 WritableDatabase::add_document(const Document & document)
 {
     LOGCALL(API, Xapian::docid, "WritableDatabase::add_document", document);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
-    RETURN(internal[0]->add_document(document));
+    size_t n_dbs = internal.size();
+    if (rare(n_dbs == 0))
+	no_subdatabases();
+    if (n_dbs == 1)
+	RETURN(internal[0]->add_document(document));
+
+    // Which database will the next never used docid be in?
+    Xapian::docid did = get_lastdocid() + 1;
+    if (rare(did == 0)) {
+	throw Xapian::DatabaseError("Run out of docids - you'll have to use copydatabase to eliminate any gaps before you can add more documents");
+    }
+    // We want exactly did to be used, not a lower docid if that subdb isn't
+    // using the docid before it, so call replace_document() not
+    // add_document().
+    size_t i = sub_db(did, n_dbs);
+    internal[i]->replace_document(sub_docid(did, n_dbs), document);
+    RETURN(did);
 }
 
 void
 WritableDatabase::delete_document(Xapian::docid did)
 {
     LOGCALL_VOID(API, "WritableDatabase::delete_document", did);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
-    if (did == 0)
+    if (rare(did == 0))
 	docid_zero_invalid();
-    internal[0]->delete_document(did);
+
+    size_t n_dbs = internal.size();
+    if (rare(n_dbs == 0))
+	no_subdatabases();
+    size_t i = sub_db(did, n_dbs);
+    internal[i]->delete_document(sub_docid(did, n_dbs));
 }
 
 void
 WritableDatabase::delete_document(const std::string & unique_term)
 {
     LOGCALL_VOID(API, "WritableDatabase::delete_document", unique_term);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
     if (unique_term.empty())
 	throw InvalidArgumentError("Empty termnames are invalid");
-    internal[0]->delete_document(unique_term);
+    size_t n_dbs = internal.size();
+    if (rare(n_dbs == 0))
+	no_subdatabases();
+    for (size_t i = 0; i != n_dbs; ++i)
+	internal[i]->delete_document(unique_term);
 }
 
 void
 WritableDatabase::replace_document(Xapian::docid did, const Document & document)
 {
     LOGCALL_VOID(API, "WritableDatabase::replace_document", did | document);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
     if (did == 0)
 	docid_zero_invalid();
-    internal[0]->replace_document(did, document);
+    size_t n_dbs = internal.size();
+    if (rare(n_dbs == 0))
+	no_subdatabases();
+    size_t i = sub_db(did, n_dbs);
+    internal[i]->replace_document(sub_docid(did, n_dbs), document);
 }
 
 Xapian::docid
@@ -853,10 +913,34 @@ WritableDatabase::replace_document(const std::string & unique_term,
 				   const Document & document)
 {
     LOGCALL(API, Xapian::docid, "WritableDatabase::replace_document", unique_term | document);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
     if (unique_term.empty())
 	throw InvalidArgumentError("Empty termnames are invalid");
-    RETURN(internal[0]->replace_document(unique_term, document));
+    size_t n_dbs = internal.size();
+    if (rare(n_dbs == 0))
+	no_subdatabases();
+    if (n_dbs == 1)
+	RETURN(internal[0]->replace_document(unique_term, document));
+
+    Xapian::PostingIterator postit = postlist_begin(unique_term);
+    // If no unique_term in the database, this is just an add_document().
+    if (postit == postlist_end(unique_term)) {
+	// Which database will the next never used docid be in?
+	size_t i = sub_db(get_lastdocid() + 1, n_dbs);
+	RETURN(internal[i]->add_document(document));
+    }
+
+    Xapian::docid retval = *postit;
+    size_t i = sub_db(retval, n_dbs);
+    internal[i]->replace_document(sub_docid(retval, n_dbs), document);
+
+    // Delete any other occurrences of unique_term.
+    while (++postit != postlist_end(unique_term)) {
+	Xapian::docid did = *postit;
+	i = sub_db(did, n_dbs);
+	internal[i]->delete_document(sub_docid(did, n_dbs));
+    }
+
+    return retval;
 }
 
 void
@@ -864,7 +948,9 @@ WritableDatabase::add_spelling(const std::string & word,
 			       Xapian::termcount freqinc) const
 {
     LOGCALL_VOID(API, "WritableDatabase::add_spelling", word | freqinc);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
+    if (rare(internal.empty()))
+	no_subdatabases();
+    // FIXME: Is adding to the first subdatabase sensible?
     internal[0]->add_spelling(word, freqinc);
 }
 
@@ -873,8 +959,12 @@ WritableDatabase::remove_spelling(const std::string & word,
 				  Xapian::termcount freqdec) const
 {
     LOGCALL_VOID(API, "WritableDatabase::remove_spelling", word | freqdec);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
-    internal[0]->remove_spelling(word, freqdec);
+    size_t n_dbs = internal.size();
+    if (rare(n_dbs == 0))
+	no_subdatabases();
+    for (size_t i = 0; i < n_dbs; ++i) {
+	internal[i]->remove_spelling(word, freqdec);
+    }
 }
 
 void
@@ -882,7 +972,9 @@ WritableDatabase::add_synonym(const std::string & term,
 			      const std::string & synonym) const
 {
     LOGCALL_VOID(API, "WritableDatabase::add_synonym", term | synonym);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
+    if (rare(internal.empty()))
+	no_subdatabases();
+    // FIXME: Is adding to the first subdatabase sensible?
     internal[0]->add_synonym(term, synonym);
 }
 
@@ -891,32 +983,41 @@ WritableDatabase::remove_synonym(const std::string & term,
 				 const std::string & synonym) const
 {
     LOGCALL_VOID(API, "WritableDatabase::remove_synonym", term | synonym);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
-    internal[0]->remove_synonym(term, synonym);
+    size_t n_dbs = internal.size();
+    if (rare(n_dbs == 0))
+	no_subdatabases();
+    for (size_t i = 0; i < n_dbs; ++i) {
+	internal[i]->remove_synonym(term, synonym);
+    }
 }
 
 void
 WritableDatabase::clear_synonyms(const std::string & term) const
 {
     LOGCALL_VOID(API, "WritableDatabase::clear_synonyms", term);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
-    internal[0]->clear_synonyms(term);
+    size_t n_dbs = internal.size();
+    if (rare(n_dbs == 0))
+	no_subdatabases();
+    for (size_t i = 0; i < n_dbs; ++i) {
+	internal[i]->clear_synonyms(term);
+    }
 }
 
 void
 WritableDatabase::set_metadata(const string & key, const string & value)
 {
     LOGCALL_VOID(API, "WritableDatabase::set_metadata", key | value);
-    if (internal.size() != 1) only_one_subdatabase_allowed();
-    if (key.empty())
-	throw InvalidArgumentError("Empty metadata keys are invalid");
+    if (rare(key.empty()))
+	empty_metadata_key();
+    if (rare(internal.empty()))
+	no_subdatabases();
     internal[0]->set_metadata(key, value);
 }
 
 string
 WritableDatabase::get_description() const
 {
-    /// \todo display contents of the writable database
+    /// @todo display contents of the writable database
     return "WritableDatabase()";
 }
 
