@@ -27,7 +27,6 @@
 #include "xapian/postingsource.h"
 #include "xapian/query.h"
 
-#include "matcher/const_database_wrapper.h"
 #include "leafpostlist.h"
 #include "matcher/andmaybepostlist.h"
 #include "matcher/andnotpostlist.h"
@@ -667,6 +666,13 @@ QueryPostingSource::QueryPostingSource(PostingSource * source_)
 {
     if (!source_)
 	throw Xapian::InvalidArgumentError("source parameter can't be NULL");
+    if (source->_refs == 0) {
+	// source_ isn't reference counted, so try to clone it.  If clone()
+	// isn't implemented, just use the object provided and it's the
+	// caller's responsibility to ensure it stays valid while in use.
+	PostingSource * cloned_source = source->clone();
+	if (cloned_source) source = cloned_source->release();
+    }
 }
 
 Query::op
@@ -735,7 +741,11 @@ QueryPostingSource::postlist(QueryOptimiser * qopt, double factor) const
     Assert(source.get());
     if (factor != 0.0)
 	qopt->inc_total_subqs();
-    Xapian::Database wrappeddb(new ConstDatabaseWrapper(&(qopt->db)));
+    // Casting away const on the Database::Internal here is OK, as we wrap
+    // them in a const Xapian::Database so non-const methods can't actually
+    // be called on the Database::Internal object.
+    const Xapian::Database wrappeddb(
+	    const_cast<Xapian::Database::Internal*>(&(qopt->db)));
     RETURN(new ExternalPostList(wrappeddb, source.get(), factor, qopt->matcher));
 }
 
@@ -950,6 +960,12 @@ QueryWildcard::postlist(QueryOptimiser * qopt, double factor) const
     } else if (op != Query::OP_SYNONYM) {
 	or_factor = factor;
     }
+
+    bool old_in_synonym = qopt->in_synonym;
+    if (!old_in_synonym) {
+	qopt->in_synonym = (op == Query::OP_SYNONYM);
+    }
+
     OrContext ctx(0);
     AutoPtr<TermList> t(qopt->db.open_allterms(pattern));
     Xapian::termcount expansions_left = max_expansion;
@@ -995,6 +1011,8 @@ QueryWildcard::postlist(QueryOptimiser * qopt, double factor) const
 	}
     }
 
+    qopt->in_synonym = old_in_synonym;
+
     if (ctx.empty())
 	RETURN(new EmptyPostList);
 
@@ -1197,6 +1215,11 @@ QueryBranch::do_synonym(QueryOptimiser * qopt, double factor) const
 {
     LOGCALL(MATCH, PostList *, "QueryBranch::do_synonym", qopt | factor);
     OrContext ctx(subqueries.size());
+    bool old_in_synonym;
+    if (factor != 0.0) {
+	old_in_synonym = qopt->in_synonym;
+	qopt->in_synonym = true;
+    }
     do_or_like(ctx, qopt, 0.0);
     PostList * pl = ctx.postlist(qopt);
     if (factor == 0.0) {
@@ -1205,6 +1228,8 @@ QueryBranch::do_synonym(QueryOptimiser * qopt, double factor) const
 	return pl;
     }
 
+    qopt->in_synonym = old_in_synonym;
+
     // We currently assume wqf is 1 for calculating the synonym's weight
     // since conceptually the synonym is one "virtual" term.  If we were
     // to combine multiple occurrences of the same synonym expansion into
