@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 Hein Ragas
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016 Olly Betts
  * Copyright 2006,2008 Lemur Consulting Ltd
  * Copyright 2009,2010 Richard Boulton
  * Copyright 2009 Kan-Ru Chen
@@ -399,7 +399,7 @@ ChertDatabase::set_revision_number(chert_revision_number_t new_revision)
     } else {
 	max_changesets = 0;
     }
- 
+
     if (max_changesets > 0) {
 	chert_revision_number_t old_revision = get_revision_number();
 	if (old_revision) {
@@ -452,7 +452,6 @@ ChertDatabase::set_revision_number(chert_revision_number_t new_revision)
 	    pack_uint(changes_tail, new_revision);
 	}
 	record_table.commit(new_revision, changes_fd, &changes_tail);
-
     } catch (...) {
 	// Remove the changeset, if there was one.
 	if (changes_fd >= 0) {
@@ -461,7 +460,7 @@ ChertDatabase::set_revision_number(chert_revision_number_t new_revision)
 
 	throw;
     }
-    
+
     if (changes_fd >= 0 && max_changesets < new_revision) {
 	// While change sets less than N - max_changesets exist, delete them
 	// 1 must be subtracted so we don't delete the changeset we just wrote
@@ -778,18 +777,6 @@ ChertDatabase::get_total_length() const
     RETURN(stats.get_total_doclen());
 }
 
-Xapian::doclength
-ChertDatabase::get_avlength() const
-{
-    LOGCALL(DB, Xapian::doclength, "ChertDatabase::get_avlength", NO_ARGS);
-    Xapian::doccount doccount = record_table.get_doccount();
-    if (doccount == 0) {
-	// Avoid dividing by zero when there are no documents.
-	RETURN(0);
-    }
-    RETURN(double(stats.get_total_doclen()) / doccount);
-}
-
 Xapian::termcount
 ChertDatabase::get_doclength(Xapian::docid did) const
 {
@@ -1081,6 +1068,25 @@ ChertWritableDatabase::commit()
     apply();
 }
 
+void
+ChertWritableDatabase::check_flush_threshold()
+{
+    // FIXME: this should be done by checking memory usage, not the number of
+    // changes.
+    // We could also look at:
+    // * mod_plists.size()
+    // * doclens.size()
+    // * freq_deltas.size()
+    //
+    // cout << "+++ mod_plists.size() " << mod_plists.size() <<
+    //     ", doclens.size() " << doclens.size() <<
+    //	   ", freq_deltas.size() " << freq_deltas.size() << endl;
+    if (++change_count >= flush_threshold) {
+	flush_postlist_changes();
+	if (!transaction_active()) apply();
+    }
+}
+
 void
 ChertWritableDatabase::flush_postlist_changes() const
 {
@@ -1235,20 +1241,7 @@ ChertWritableDatabase::add_document_(Xapian::docid did,
 	throw;
     }
 
-    // FIXME: this should be done by checking memory usage, not the number of
-    // changes.
-    // We could also look at:
-    // * mod_plists.size()
-    // * doclens.size()
-    // * freq_deltas.size()
-    //
-    // cout << "+++ mod_plists.size() " << mod_plists.size() <<
-    //     ", doclens.size() " << doclens.size() <<
-    //	   ", freq_deltas.size() " << freq_deltas.size() << endl;
-    if (++change_count >= flush_threshold) {
-	flush_postlist_changes();
-	if (!transaction_active()) apply();
-    }
+    check_flush_threshold();
 
     RETURN(did);
 }
@@ -1311,10 +1304,7 @@ ChertWritableDatabase::delete_document(Xapian::docid did)
 	throw;
     }
 
-    if (++change_count >= flush_threshold) {
-	flush_postlist_changes();
-	if (!transaction_active()) apply();
-    }
+    check_flush_threshold();
 }
 
 void
@@ -1433,7 +1423,7 @@ ChertWritableDatabase::replace_document(Xapian::docid did,
 		    stats.check_wdf(new_wdf);
 
 		    if (old_wdf != new_wdf) {
-		    	new_doclen += new_wdf - old_wdf;
+			new_doclen += new_wdf - old_wdf;
 			add_freq_delta(new_tname, 0, new_wdf - old_wdf);
 			update_mod_plist(did, new_tname, 'M', new_wdf);
 		    }
@@ -1486,10 +1476,7 @@ ChertWritableDatabase::replace_document(Xapian::docid did,
 	throw;
     }
 
-    if (++change_count >= flush_threshold) {
-	flush_postlist_changes();
-	if (!transaction_active()) apply();
-    }
+    check_flush_threshold();
 }
 
 Xapian::Document::Internal *
