@@ -115,15 +115,15 @@ static inline byte *zeroed_new(size_t size)
    And then,
 
    R = REVISION(b)  is the revision number the B-tree had when the block was
-                    written into the DB file.
+		    written into the DB file.
    L = GET_LEVEL(b) is the level of the block, which is the number of levels
-                    towards the root of the B-tree structure. So leaf blocks
-                    have level 0 and the one root block has the highest level
-                    equal to the number of levels in the B-tree.
+		    towards the root of the B-tree structure. So leaf blocks
+		    have level 0 and the one root block has the highest level
+		    equal to the number of levels in the B-tree.
    M = MAX_FREE(b)  is the size of the gap between the end of the directory and
-                    the first item of data. (It is not necessarily the maximum
-                    size among the bits of space that are free, but I can't
-                    think of a better name.)
+		    the first item of data. (It is not necessarily the maximum
+		    size among the bits of space that are free, but I can't
+		    think of a better name.)
    T = TOTAL_FREE(b)is the total amount of free space left in b.
    D = DIR_END(b)   gives the offset to the end of the directory.
 
@@ -133,9 +133,9 @@ static inline byte *zeroed_new(size_t size)
 
    An item has this form:
 
-           I K key x C tag
-             <--K-->
-           <------I------>
+	   I K key x C tag
+	     <--K-->
+	   <------I------>
 
    A long tag presented through the API is split up into C tags small enough to
    be accommodated in the blocks of the B-tree. The key is extended to include
@@ -1000,14 +1000,15 @@ ChertTable::add(const string &key, string tag, bool already_compressed)
 
 	lazy_alloc_deflate_zstream();
 
-	deflate_zstream->next_in = (Bytef *)const_cast<char *>(tag.data());
-	deflate_zstream->avail_in = (uInt)tag.size();
+	deflate_zstream->next_in =
+	    reinterpret_cast<Bytef *>(const_cast<char *>(tag.data()));
+	deflate_zstream->avail_in = static_cast<uInt>(tag.size());
 
 	// If compressed size is >= tag.size(), we don't want to compress.
 	unsigned long blk_len = tag.size() - 1;
 	unsigned char * blk = new unsigned char[blk_len];
 	deflate_zstream->next_out = blk;
-	deflate_zstream->avail_out = (uInt)blk_len;
+	deflate_zstream->avail_out = static_cast<uInt>(blk_len);
 
 	int err = deflate(deflate_zstream, Z_FINISH);
 	if (err == Z_STREAM_END) {
@@ -1252,13 +1253,14 @@ ChertTable::read_tag(Cursor * C_, string *tag, bool keep_compressed) const
 
     lazy_alloc_inflate_zstream();
 
-    inflate_zstream->next_in = (Bytef*)const_cast<char *>(tag->data());
-    inflate_zstream->avail_in = (uInt)tag->size();
+    inflate_zstream->next_in =
+	reinterpret_cast<Bytef*>(const_cast<char *>(tag->data()));
+    inflate_zstream->avail_in = static_cast<uInt>(tag->size());
 
     int err = Z_OK;
     while (err != Z_STREAM_END) {
 	inflate_zstream->next_out = buf;
-	inflate_zstream->avail_out = (uInt)sizeof(buf);
+	inflate_zstream->avail_out = static_cast<uInt>(sizeof(buf));
 	err = inflate(inflate_zstream, Z_SYNC_FLUSH);
 	if (err == Z_BUF_ERROR && inflate_zstream->avail_in == 0) {
 	    LOGLINE(DB, "Z_BUF_ERROR - faking checksum of " << inflate_zstream->adler);
@@ -1289,7 +1291,7 @@ ChertTable::read_tag(Cursor * C_, string *tag, bool keep_compressed) const
 	msg += str(utag.size());
 	msg += " != ";
 	// OpenBSD's zlib.h uses off_t instead of uLong for total_out.
-	msg += str((size_t)inflate_zstream->total_out);
+	msg += str(size_t(inflate_zstream->total_out));
 	throw Xapian::DatabaseCorruptError(msg);
     }
 
