@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2014 Olly Betts
  * Copyright 2006,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -27,11 +27,11 @@
 
 #include "debuglog.h"
 
-#include "expandweight.h"
+#include "expand/expandweight.h"
 #include "inmemory_document.h"
 #include "inmemory_alltermslist.h"
 #include "str.h"
-#include "valuestats.h"
+#include "backends/valuestats.h"
 
 #include <string>
 #include <vector>
@@ -41,6 +41,7 @@
 #include <xapian/valueiterator.h>
 
 using std::make_pair;
+using Xapian::Internal::intrusive_ptr;
 
 inline void
 InMemoryTerm::add_posting(const InMemoryPosting & post)
@@ -76,7 +77,7 @@ InMemoryDoc::add_posting(const InMemoryTermEntry & post)
 // Postlist //
 //////////////
 
-InMemoryPostList::InMemoryPostList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db_,
+InMemoryPostList::InMemoryPostList(intrusive_ptr<const InMemoryDatabase> db_,
 				   const InMemoryTerm & imterm,
 				   const std::string & term_)
 	: LeafPostList(term_),
@@ -105,7 +106,7 @@ InMemoryPostList::get_docid() const
 }
 
 PostList *
-InMemoryPostList::next(Xapian::weight /*w_min*/)
+InMemoryPostList::next(double /*w_min*/)
 {
     if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     if (started) {
@@ -119,7 +120,7 @@ InMemoryPostList::next(Xapian::weight /*w_min*/)
 }
 
 PostList *
-InMemoryPostList::skip_to(Xapian::docid did, Xapian::weight w_min)
+InMemoryPostList::skip_to(Xapian::docid did, double w_min)
 {
     if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     // FIXME - see if we can make more efficient, perhaps using better
@@ -156,6 +157,12 @@ InMemoryPostList::get_doclength() const
     return db->get_doclength(get_docid());
 }
 
+Xapian::termcount
+InMemoryPostList::get_unique_terms() const
+{
+    return db->get_unique_terms(get_docid());
+}
+
 PositionList *
 InMemoryPostList::read_position_list()
 {
@@ -182,7 +189,7 @@ InMemoryPostList::get_wdf() const
 // Termlist //
 //////////////
 
-InMemoryTermList::InMemoryTermList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db_,
+InMemoryTermList::InMemoryTermList(intrusive_ptr<const InMemoryDatabase> db_,
 				   Xapian::docid did_,
 				   const InMemoryDoc & doc,
 				   Xapian::termcount len)
@@ -190,7 +197,7 @@ InMemoryTermList::InMemoryTermList(Xapian::Internal::RefCntPtr<const InMemoryDat
 	  started(false), db(db_), did(did_), document_length(len)
 {
     LOGLINE(DB, "InMemoryTermList::InMemoryTermList(): " <<
-	        terms << " terms starting from " << pos->tname);
+		terms << " terms starting from " << pos->tname);
 }
 
 Xapian::termcount
@@ -209,7 +216,9 @@ InMemoryTermList::get_termfreq() const
     Assert(started);
     Assert(!at_end());
 
-    return db->get_termfreq((*pos).tname);
+    Xapian::doccount tf;
+    db->get_freqs((*pos).tname, &tf, NULL);
+    return tf;
 }
 
 Xapian::termcount
@@ -292,7 +301,7 @@ InMemoryTermList::positionlist_begin() const
 // InMemoryAllDocsPostList //
 /////////////////////////////
 
-InMemoryAllDocsPostList::InMemoryAllDocsPostList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db_)
+InMemoryAllDocsPostList::InMemoryAllDocsPostList(intrusive_ptr<const InMemoryDatabase> db_)
 	: LeafPostList(std::string()), did(0), db(db_)
 {
 }
@@ -321,6 +330,12 @@ InMemoryAllDocsPostList::get_doclength() const
     return db->get_doclength(did);
 }
 
+Xapian::termcount
+InMemoryAllDocsPostList::get_unique_terms() const
+{
+    return db->get_unique_terms(did);
+}
+
 Xapian::termcount
 InMemoryAllDocsPostList::get_wdf() const
 {
@@ -340,7 +355,7 @@ InMemoryAllDocsPostList::open_position_list() const
 }
 
 PostList *
-InMemoryAllDocsPostList::next(Xapian::weight /*w_min*/)
+InMemoryAllDocsPostList::next(double /*w_min*/)
 {
     if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(!at_end());
@@ -351,7 +366,7 @@ InMemoryAllDocsPostList::next(Xapian::weight /*w_min*/)
 }
 
 PostList *
-InMemoryAllDocsPostList::skip_to(Xapian::docid did_, Xapian::weight /*w_min*/)
+InMemoryAllDocsPostList::skip_to(Xapian::docid did_, double /*w_min*/)
 {
     if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(!at_end());
@@ -397,10 +412,11 @@ InMemoryDatabase::~InMemoryDatabase()
     dtor_called();
 }
 
-void
+bool
 InMemoryDatabase::reopen()
 {
     if (closed) InMemoryDatabase::throw_database_closed();
+    return false;
 }
 
 void
@@ -422,7 +438,7 @@ InMemoryDatabase::open_post_list(const string & tname) const
 {
     if (closed) InMemoryDatabase::throw_database_closed();
     if (tname.empty()) {
-	Xapian::Internal::RefCntPtr<const InMemoryDatabase> ptrtothis(this);
+	intrusive_ptr<const InMemoryDatabase> ptrtothis(this);
 	return new InMemoryAllDocsPostList(ptrtothis);
     }
     map<string, InMemoryTerm>::const_iterator i = postlists.find(tname);
@@ -431,7 +447,7 @@ InMemoryDatabase::open_post_list(const string & tname) const
 	// Check that our dummy entry for string() is present.
 	Assert(i->first.empty());
     }
-    Xapian::Internal::RefCntPtr<const InMemoryDatabase> ptrtothis(this);
+    intrusive_ptr<const InMemoryDatabase> ptrtothis(this);
     return new InMemoryPostList(ptrtothis, i->second, tname);
 }
 
@@ -442,22 +458,24 @@ InMemoryDatabase::doc_exists(Xapian::docid did) const
     return (did > 0 && did <= termlists.size() && termlists[did - 1].is_valid);
 }
 
-Xapian::doccount
-InMemoryDatabase::get_termfreq(const string & tname) const
+void
+InMemoryDatabase::get_freqs(const string & term,
+			    Xapian::doccount * termfreq_ptr,
+			    Xapian::termcount * collfreq_ptr) const
 {
     if (closed) InMemoryDatabase::throw_database_closed();
-    map<string, InMemoryTerm>::const_iterator i = postlists.find(tname);
-    if (i == postlists.end()) return 0;
-    return i->second.term_freq;
-}
-
-Xapian::termcount
-InMemoryDatabase::get_collection_freq(const string &tname) const
-{
-    if (closed) InMemoryDatabase::throw_database_closed();
-    map<string, InMemoryTerm>::const_iterator i = postlists.find(tname);
-    if (i == postlists.end()) return 0;
-    return i->second.collection_freq;
+    map<string, InMemoryTerm>::const_iterator i = postlists.find(term);
+    if (i != postlists.end()) {
+	if (termfreq_ptr)
+	    *termfreq_ptr = i->second.term_freq;
+	if (collfreq_ptr)
+	    *collfreq_ptr = i->second.collection_freq;
+    } else {
+	if (termfreq_ptr)
+	    *termfreq_ptr = 0;
+	if (collfreq_ptr)
+	    *collfreq_ptr = 0;
+    }
 }
 
 Xapian::doccount
@@ -507,14 +525,6 @@ InMemoryDatabase::get_total_length() const
     return totlen;
 }
 
-Xapian::doclength
-InMemoryDatabase::get_avlength() const
-{
-    if (closed) InMemoryDatabase::throw_database_closed();
-    if (totdocs == 0) return 0;
-    return Xapian::doclength(totlen) / totdocs;
-}
-
 Xapian::termcount
 InMemoryDatabase::get_doclength(Xapian::docid did) const
 {
@@ -526,6 +536,16 @@ InMemoryDatabase::get_doclength(Xapian::docid did) const
     return doclengths[did - 1];
 }
 
+Xapian::termcount
+InMemoryDatabase::get_unique_terms(Xapian::docid did) const
+{
+    if (closed) InMemoryDatabase::throw_database_closed();
+    if (did == 0 || did > termlists.size() || !termlists[did - 1].is_valid)
+	throw Xapian::DocNotFoundError(string("Docid ") + str(did) +
+				 string(" not found"));
+    return termlists[did - 1].terms.size();
+}
+
 TermList *
 InMemoryDatabase::open_term_list(Xapian::docid did) const
 {
@@ -536,7 +556,7 @@ InMemoryDatabase::open_term_list(Xapian::docid did) const
 	throw Xapian::DocNotFoundError(string("Docid ") + str(did) +
 				 string(" not found"));
     }
-    return new InMemoryTermList(Xapian::Internal::RefCntPtr<const InMemoryDatabase>(this), did,
+    return new InMemoryTermList(intrusive_ptr<const InMemoryDatabase>(this), did,
 				termlists[did - 1], doclengths[did - 1]);
 }
 
@@ -563,11 +583,12 @@ InMemoryDatabase::get_metadata(const std::string & key) const
 	return string();
     return i->second;
 }
- 
+
 TermList *
 InMemoryDatabase::open_metadata_keylist(const string &) const
 {
-    if (!closed && metadata.empty()) return NULL;
+    if (closed) InMemoryDatabase::throw_database_closed();
+    if (metadata.empty()) return NULL;
     // FIXME: nobody implemented this yet...
     throw Xapian::UnimplementedError("InMemory backend doesn't currently implement Database::metadata_keys_begin()");
 }
@@ -603,7 +624,7 @@ InMemoryDatabase::positionlist_count(Xapian::docid did,
     return 0;
 }
 
-PositionList * 
+PositionList *
 InMemoryDatabase::open_position_list(Xapian::docid did,
 				     const string & tname) const
 {
@@ -723,7 +744,7 @@ InMemoryDatabase::replace_document(Xapian::docid did,
 
     if (closed) InMemoryDatabase::throw_database_closed();
 
-    if (doc_exists(did)) { 
+    if (doc_exists(did)) {
 	map<Xapian::valueno, string>::const_iterator j;
 	for (j = valuelists[did-1].begin(); j != valuelists[did-1].end(); ++j) {
 	    map<Xapian::valueno, ValueStats>::iterator i;
@@ -799,20 +820,17 @@ InMemoryDatabase::finish_add_doc(Xapian::docid did, const Xapian::Document &docu
 
     InMemoryDoc doc(true);
     Xapian::TermIterator i = document.termlist_begin();
-    Xapian::TermIterator i_end = document.termlist_end();
-    for ( ; i != i_end; ++i) {
+    for ( ; i != document.termlist_end(); ++i) {
 	make_term(*i);
 
 	LOGLINE(DB, "InMemoryDatabase::finish_add_doc(): adding term " << *i);
 	Xapian::PositionIterator j = i.positionlist_begin();
-	Xapian::PositionIterator j_end = i.positionlist_end();
-
-	if (j == j_end) {
+	if (j == i.positionlist_end()) {
 	    /* Make sure the posting exists, even without a position. */
 	    make_posting(&doc, *i, did, 0, i.get_wdf(), false);
 	} else {
 	    positions_present = true;
-	    for ( ; j != j_end; ++j) {
+	    for ( ; j != i.positionlist_end(); ++j) {
 		make_posting(&doc, *i, did, *j, i.get_wdf());
 	    }
 	}
@@ -905,7 +923,7 @@ InMemoryDatabase::open_allterms(const string & prefix) const
 {
     if (closed) InMemoryDatabase::throw_database_closed();
     return new InMemoryAllTermsList(&postlists,
-				    Xapian::Internal::RefCntPtr<const InMemoryDatabase>(this),
+				    intrusive_ptr<const InMemoryDatabase>(this),
 				    prefix);
 }
 
