@@ -1,8 +1,9 @@
-/* inmemory_database.h: C++ class definition for inmemory database access
- *
- * Copyright 1999,2000,2001 BrightStation PLC
+/** @file inmemory_database.h
+ * @brief C++ class definition for inmemory database access
+ */
+/* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2014 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2014,2015 Olly Betts
  * Copyright 2006,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -24,10 +25,11 @@
 #ifndef OM_HGUARD_INMEMORY_DATABASE_H
 #define OM_HGUARD_INMEMORY_DATABASE_H
 
-#include "leafpostlist.h"
-#include "termlist.h"
-#include "database.h"
-#include "valuestats.h"
+#include "api/leafpostlist.h"
+#include "api/termlist.h"
+#include "backends/backends.h"
+#include "backends/database.h"
+#include "backends/valuestats.h"
 #include <map>
 #include <vector>
 #include <algorithm>
@@ -144,22 +146,23 @@ class InMemoryPostList : public LeafPostList {
 	 */
 	InMemoryPositionList mypositions;
 
-	Xapian::Internal::RefCntPtr<const InMemoryDatabase> db;
+	Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db;
 
-	InMemoryPostList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db,
+	InMemoryPostList(Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db,
 			 const InMemoryTerm & imterm, const std::string & term_);
     public:
 	Xapian::doccount get_termfreq() const;
 
 	Xapian::docid       get_docid() const;     // Gets current docid
 	Xapian::termcount   get_doclength() const; // Length of current document
+	Xapian::termcount   get_unique_terms() const; // number of terms in current document
 	Xapian::termcount   get_wdf() const;	   // Within Document Frequency
 	PositionList * read_position_list();
 	PositionList * open_position_list() const;
 
-	PostList *next(Xapian::weight w_min); // Moves to next docid
+	PostList *next(double w_min); // Moves to next docid
 
-	PostList *skip_to(Xapian::docid did, Xapian::weight w_min); // Moves to next docid >= specified docid
+	PostList *skip_to(Xapian::docid did, double w_min); // Moves to next docid >= specified docid
 
 	// True if we're off the end of the list.
 	bool at_end() const;
@@ -174,21 +177,22 @@ class InMemoryAllDocsPostList : public LeafPostList {
     private:
 	Xapian::docid did;
 
-	Xapian::Internal::RefCntPtr<const InMemoryDatabase> db;
+	Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db;
 
-	InMemoryAllDocsPostList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db);
+	InMemoryAllDocsPostList(Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db);
     public:
 	Xapian::doccount get_termfreq() const;
 
 	Xapian::docid       get_docid() const;     // Gets current docid
 	Xapian::termcount   get_doclength() const; // Length of current document
+	Xapian::termcount   get_unique_terms() const; // number of terms in current document
 	Xapian::termcount   get_wdf() const;       // Within Document Frequency
 	PositionList * read_position_list();
 	PositionList * open_position_list() const;
 
-	PostList *next(Xapian::weight w_min);      // Moves to next docid
+	PostList *next(double w_min);      // Moves to next docid
 
-	PostList *skip_to(Xapian::docid did, Xapian::weight w_min); // Moves to next docid >= specified docid
+	PostList *skip_to(Xapian::docid did, double w_min); // Moves to next docid >= specified docid
 
 	// True if we're off the end of the list
 	bool at_end() const;
@@ -205,11 +209,11 @@ class InMemoryTermList : public TermList {
 	Xapian::termcount terms;
 	bool started;
 
-	Xapian::Internal::RefCntPtr<const InMemoryDatabase> db;
+	Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db;
 	Xapian::docid did;
 	Xapian::termcount document_length;
 
-	InMemoryTermList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db,
+	InMemoryTermList(Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db,
 			 Xapian::docid did,
 			 const InMemoryDoc & doc,
 			 Xapian::termcount len);
@@ -307,7 +311,7 @@ class InMemoryDatabase : public Xapian::Database::Internal {
 
     ~InMemoryDatabase();
 
-    void reopen();
+    bool reopen();
     void close();
     bool is_closed() const { return closed; }
 
@@ -316,11 +320,12 @@ class InMemoryDatabase : public Xapian::Database::Internal {
     Xapian::docid get_lastdocid() const;
 
     totlen_t get_total_length() const;
-    Xapian::doclength get_avlength() const;
     Xapian::termcount get_doclength(Xapian::docid did) const;
+    Xapian::termcount get_unique_terms(Xapian::docid did) const;
 
-    Xapian::doccount get_termfreq(const string & tname) const;
-    Xapian::termcount get_collection_freq(const string & tname) const;
+    void get_freqs(const string & term,
+		   Xapian::doccount * termfreq_ptr,
+		   Xapian::termcount * collfreq_ptr) const;
     Xapian::doccount get_value_freq(Xapian::valueno slot) const;
     std::string get_value_lower_bound(Xapian::valueno slot) const;
     std::string get_value_upper_bound(Xapian::valueno slot) const;
@@ -342,6 +347,11 @@ class InMemoryDatabase : public Xapian::Database::Internal {
     TermList * open_allterms(const string & prefix) const;
 
     XAPIAN_NORETURN(static void throw_database_closed());
+
+    int get_backend_info(string * path) const {
+	if (path) *path = string();
+	return BACKEND_INMEMORY;
+    }
 };
 
 #endif /* OM_HGUARD_INMEMORY_DATABASE_H */
