@@ -1,7 +1,7 @@
 /** @file remote-database.cc
  *  @brief Remote backend database class
  */
-/* Copyright (C) 2006,2007,2008,2009,2010,2015 Olly Betts
+/* Copyright (C) 2006,2007,2008,2009,2010,2011,2012,2013,2014,2015 Olly Betts
  * Copyright (C) 2007,2009,2010 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -27,29 +27,49 @@
 #include <signal.h>
 
 #include "autoptr.h"
-#include "emptypostlist.h"
-#include "inmemory_positionlist.h"
+#include "api/emptypostlist.h"
+#include "backends/inmemory/inmemory_positionlist.h"
 #include "net_postlist.h"
 #include "net_termlist.h"
+#include "noreturn.h"
 #include "remote-document.h"
 #include "omassert.h"
 #include "realtime.h"
-#include "serialise.h"
+#include "net/length.h"
+#include "net/serialise.h"
+#include "net/serialise-error.h"
 #include "serialise-double.h"
 #include "str.h"
 #include "stringutils.h" // For STRINGIZE().
-#include "weightinternal.h"
+#include "weight/weightinternal.h"
 
 #include <string>
 #include <vector>
 
+#include "xapian/constants.h"
 #include "xapian/error.h"
 #include "xapian/matchspy.h"
 
 using namespace std;
+using Xapian::Internal::intrusive_ptr;
+
+XAPIAN_NORETURN(static void throw_bad_message(const string & context));
+static void
+throw_bad_message(const string & context)
+{
+    throw Xapian::NetworkError("Bad message received", context);
+}
+
+XAPIAN_NORETURN(static void throw_connection_closed_unexpectedly());
+static void
+throw_connection_closed_unexpectedly()
+{
+    throw Xapian::NetworkError("Connection closed unexpectedly");
+}
 
 RemoteDatabase::RemoteDatabase(int fd, double timeout_,
-			       const string & context_, bool writable)
+			       const string & context_, bool writable,
+			       int flags)
 	: link(fd, fd, context_),
 	  context(context_),
 	  cached_stats_valid(),
@@ -74,45 +94,16 @@ RemoteDatabase::RemoteDatabase(int fd, double timeout_,
 	transaction_state = TRANSACTION_UNIMPLEMENTED;
     }
 
-    string message;
-    char type = get_message(message);
+    update_stats(MSG_MAX);
 
-    if (reply_type(type) != REPLY_GREETING || message.size() < 3) {
-	if (type == 'O' && message.size() == size_t('M') && message[0] == ' ') {
-	    // The server reply used to start "OM ", which will now be
-	    // interpreted as a type 'O' message of length size_t('M')
-	    // with first character ' '.
-	    throw Xapian::NetworkError("Server protocol version too old", context);
+    if (writable) {
+	if (flags & Xapian::DB_RETRY_LOCK) {
+	    const string & body = encode_length(flags & Xapian::DB_RETRY_LOCK);
+	    update_stats(MSG_WRITEACCESS, body);
+	} else {
+	    update_stats(MSG_WRITEACCESS);
 	}
-	throw Xapian::NetworkError("Handshake failed - is this a Xapian server?", context);
     }
-
-    const char *p = message.c_str();
-    const char *p_end = p + message.size();
-
-    // The protocol major versions must match.  The protocol minor version of
-    // the server must be >= that of the client.
-    int protocol_major = static_cast<unsigned char>(*p++);
-    int protocol_minor = static_cast<unsigned char>(*p++);
-    if (protocol_major != XAPIAN_REMOTE_PROTOCOL_MAJOR_VERSION ||
-	protocol_minor < XAPIAN_REMOTE_PROTOCOL_MINOR_VERSION) {
-	string errmsg("Unknown protocol version ");
-	errmsg += str(protocol_major);
-	errmsg += '.';
-	errmsg += str(protocol_minor);
-	errmsg += " (" STRINGIZE(XAPIAN_REMOTE_PROTOCOL_MAJOR_VERSION) "." STRINGIZE(XAPIAN_REMOTE_PROTOCOL_MINOR_VERSION) " supported)";
-	throw Xapian::NetworkError(errmsg, context);
-    }
-
-    apply_stats_update(p, p_end);
-
-    if (writable) update_stats(MSG_WRITEACCESS);
-}
-
-RemoteDatabase *
-RemoteDatabase::as_remotedatabase()
-{
-    return this;
 }
 
 void
@@ -134,19 +125,21 @@ RemoteDatabase::open_metadata_keylist(const std::string &prefix) const
     string message;
     AutoPtr<NetworkTermList> tlist(
 	new NetworkTermList(0, doccount,
-			    Xapian::Internal::RefCntPtr<const RemoteDatabase>(this),
+			    intrusive_ptr<const RemoteDatabase>(this),
 			    0));
     vector<NetworkTermListItem> & items = tlist->items;
 
+    string term = prefix;
     char type;
     while ((type = get_message(message)) == REPLY_METADATAKEYLIST) {
 	NetworkTermListItem item;
-	item.tname = message;
+	term.resize(size_t(static_cast<unsigned char>(message[0])));
+	term.append(message, 1, string::npos);
+	item.tname = term;
 	items.push_back(item);
     }
-    if (type != REPLY_DONE) {
-	throw Xapian::NetworkError("Bad message received", context);
-    }
+    if (type != REPLY_DONE)
+	throw_bad_message(context);
 
     tlist->current_position = tlist->items.begin();
     return tlist.release();
@@ -166,30 +159,33 @@ RemoteDatabase::open_term_list(Xapian::docid did) const
     get_message(message, REPLY_DOCLENGTH);
     const char * p = message.c_str();
     const char * p_end = p + message.size();
-    Xapian::termcount doclen = decode_length(&p, p_end, false);
+    Xapian::termcount doclen;
+    decode_length(&p, p_end, doclen);
     if (p != p_end) {
 	throw Xapian::NetworkError("Bad REPLY_DOCLENGTH message received", context);
     }
 
     AutoPtr<NetworkTermList> tlist(
 	new NetworkTermList(doclen, doccount,
-			    Xapian::Internal::RefCntPtr<const RemoteDatabase>(this),
+			    intrusive_ptr<const RemoteDatabase>(this),
 			    did));
     vector<NetworkTermListItem> & items = tlist->items;
 
+    string term;
     char type;
     while ((type = get_message(message)) == REPLY_TERMLIST) {
 	NetworkTermListItem item;
 	p = message.data();
 	p_end = p + message.size();
-	item.wdf = decode_length(&p, p_end, false);
-	item.termfreq = decode_length(&p, p_end, false);
-	item.tname.assign(p, p_end);
+	decode_length(&p, p_end, item.wdf);
+	decode_length(&p, p_end, item.termfreq);
+	term.resize(size_t(static_cast<unsigned char>(*p++)));
+	term.append(p, p_end);
+	item.tname = term;
 	items.push_back(item);
     }
-    if (type != REPLY_DONE) {
-	throw Xapian::NetworkError("Bad message received", context);
-    }
+    if (type != REPLY_DONE)
+	throw_bad_message(context);
 
     tlist->current_position = tlist->items.begin();
     return tlist.release();
@@ -204,23 +200,25 @@ RemoteDatabase::open_allterms(const string & prefix) const {
 
     AutoPtr<NetworkTermList> tlist(
 	new NetworkTermList(0, doccount,
-			    Xapian::Internal::RefCntPtr<const RemoteDatabase>(this),
+			    intrusive_ptr<const RemoteDatabase>(this),
 			    0));
     vector<NetworkTermListItem> & items = tlist->items;
 
+    string term = prefix;
     string message;
     char type;
     while ((type = get_message(message)) == REPLY_ALLTERMS) {
 	NetworkTermListItem item;
 	const char * p = message.data();
 	const char * p_end = p + message.size();
-	item.termfreq = decode_length(&p, p_end, false);
-	item.tname.assign(p, p_end);
+	decode_length(&p, p_end, item.termfreq);
+	term.resize(size_t(static_cast<unsigned char>(*p++)));
+	term.append(p, p_end);
+	item.tname = term;
 	items.push_back(item);
     }
-    if (type != REPLY_DONE) {
-	throw Xapian::NetworkError("Bad message received", context);
-    }
+    if (type != REPLY_DONE)
+	throw_bad_message(context);
 
     tlist->current_position = tlist->items.begin();
     return tlist.release();
@@ -229,7 +227,7 @@ RemoteDatabase::open_allterms(const string & prefix) const {
 LeafPostList *
 RemoteDatabase::open_post_list(const string &term) const
 {
-    return new NetworkPostList(Xapian::Internal::RefCntPtr<const RemoteDatabase>(this), term);
+    return new NetworkPostList(intrusive_ptr<const RemoteDatabase>(this), term);
 }
 
 Xapian::doccount
@@ -243,14 +241,14 @@ RemoteDatabase::read_post_list(const string &term, NetworkPostList & pl) const
 
     const char * p = message.data();
     const char * p_end = p + message.size();
-    Xapian::doccount termfreq = decode_length(&p, p_end, false);
+    Xapian::doccount termfreq;
+    decode_length(&p, p_end, termfreq);
 
     while ((type = get_message(message)) == REPLY_POSTLISTITEM) {
 	pl.append_posting(message);
     }
-    if (type != REPLY_DONE) {
-	throw Xapian::NetworkError("Bad message received", context);
-    }
+    if (type != REPLY_DONE)
+	throw_bad_message(context);
 
     return termfreq;
 }
@@ -268,12 +266,13 @@ RemoteDatabase::open_position_list(Xapian::docid did, const string &term) const
     while ((type = get_message(message)) == REPLY_POSITIONLIST) {
 	const char * p = message.data();
 	const char * p_end = p + message.size();
-	lastpos += decode_length(&p, p_end, false) + 1;
+	Xapian::termpos inc;
+	decode_length(&p, p_end, inc);
+	lastpos += inc + 1;
 	positions.push_back(lastpos);
     }
-    if (type != REPLY_DONE) {
-	throw Xapian::NetworkError("Bad message received", context);
-    }
+    if (type != REPLY_DONE)
+	throw_bad_message(context);
 
     return new InMemoryPositionList(positions);
 }
@@ -285,11 +284,11 @@ RemoteDatabase::has_positions() const
     return has_positional_info;
 }
 
-void
+bool
 RemoteDatabase::reopen()
 {
-    update_stats(MSG_REOPEN);
     mru_slot = Xapian::BAD_VALUENO;
+    return update_stats(MSG_REOPEN);
 }
 
 void
@@ -320,41 +319,74 @@ RemoteDatabase::open_document(Xapian::docid did, bool /*lazy*/) const
     while ((type = get_message(message)) == REPLY_VALUE) {
 	const char * p = message.data();
 	const char * p_end = p + message.size();
-	Xapian::valueno slot = decode_length(&p, p_end, false);
+	Xapian::valueno slot;
+	decode_length(&p, p_end, slot);
 	values.insert(make_pair(slot, string(p, p_end)));
     }
-    if (type != REPLY_DONE) {
-	throw Xapian::NetworkError("Bad message received", context);
-    }
+    if (type != REPLY_DONE)
+	throw_bad_message(context);
 
     return new RemoteDocument(this, did, doc_data, values);
 }
 
-void
-RemoteDatabase::update_stats(message_type msg_code) const
+bool
+RemoteDatabase::update_stats(message_type msg_code, const string & body) const
 {
-    send_message(msg_code, string());
-    string message;
-    get_message(message, REPLY_UPDATE);
-    const char * p = message.c_str();
-    const char * p_end = p + message.size();
-    apply_stats_update(p, p_end);
-}
+    // MSG_MAX signals that we're handling the opening greeting, which isn't in
+    // response to an explicit message.
+    if (msg_code != MSG_MAX)
+	send_message(msg_code, body);
 
-void
-RemoteDatabase::apply_stats_update(const char * p, const char * p_end) const
-{
-    doccount = decode_length(&p, p_end, false);
-    lastdocid = decode_length(&p, p_end, false);
-    doclen_lbound = decode_length(&p, p_end, false);
-    doclen_ubound = decode_length(&p, p_end, false);
+    string message;
+    reply_type type = get_message(message);
+    if (type != REPLY_UPDATE || message.size() < 3) {
+	if (type == REPLY_DONE) {
+	    // The database was already open at the latest revision.
+	    return false;
+	}
+	throw Xapian::NetworkError("Handshake failed - is this a Xapian server?", context);
+    }
+
+    const char *p = message.c_str();
+    const char *p_end = p + message.size();
+
+    // The protocol major versions must match.  The protocol minor version of
+    // the server must be >= that of the client.
+    int protocol_major = static_cast<unsigned char>(*p++);
+    int protocol_minor = static_cast<unsigned char>(*p++);
+    if (protocol_major != XAPIAN_REMOTE_PROTOCOL_MAJOR_VERSION ||
+	protocol_minor < XAPIAN_REMOTE_PROTOCOL_MINOR_VERSION) {
+	string errmsg("Server supports protocol version");
+	if (protocol_minor) {
+	    errmsg += "s ";
+	    errmsg += str(protocol_major);
+	    errmsg += ".0 to ";
+	}
+	errmsg += str(protocol_major);
+	errmsg += '.';
+	errmsg += str(protocol_minor);
+	errmsg +=
+	    " - client is using "
+	    STRINGIZE(XAPIAN_REMOTE_PROTOCOL_MAJOR_VERSION)
+	    "."
+	    STRINGIZE(XAPIAN_REMOTE_PROTOCOL_MINOR_VERSION);
+	throw Xapian::NetworkError(errmsg, context);
+    }
+
+    decode_length(&p, p_end, doccount);
+    decode_length(&p, p_end, lastdocid);
+    lastdocid += doccount;
+    decode_length(&p, p_end, doclen_lbound);
+    decode_length(&p, p_end, doclen_ubound);
+    doclen_ubound += doclen_lbound;
     if (p == p_end) {
 	throw Xapian::NetworkError("Bad stats update message received", context);
     }
     has_positional_info = (*p++ == '1');
-    total_length = decode_length(&p, p_end, false);
+    decode_length(&p, p_end, total_length);
     uuid.assign(p, p_end);
     cached_stats_valid = true;
+    return true;
 }
 
 Xapian::doccount
@@ -378,13 +410,6 @@ RemoteDatabase::get_total_length() const
     return total_length;
 }
 
-Xapian::doclength
-RemoteDatabase::get_avlength() const
-{
-    if (!cached_stats_valid) update_stats();
-    return Xapian::doclength(total_length) / doccount;
-}
-
 bool
 RemoteDatabase::term_exists(const string & tname) const
 {
@@ -392,37 +417,42 @@ RemoteDatabase::term_exists(const string & tname) const
     send_message(MSG_TERMEXISTS, tname);
     string message;
     reply_type type = get_message(message);
-    if (type != REPLY_TERMEXISTS && type != REPLY_TERMDOESNTEXIST) {
-	throw Xapian::NetworkError("Bad message received", context);
-    }
+    if (type != REPLY_TERMEXISTS && type != REPLY_TERMDOESNTEXIST)
+	throw_bad_message(context);
     return (type == REPLY_TERMEXISTS);
 }
 
-Xapian::doccount
-RemoteDatabase::get_termfreq(const string & tname) const
+void
+RemoteDatabase::get_freqs(const string & term,
+			  Xapian::doccount * termfreq_ptr,
+			  Xapian::termcount * collfreq_ptr) const
 {
-    Assert(!tname.empty());
-    send_message(MSG_TERMFREQ, tname);
+    Assert(!term.empty());
     string message;
-    get_message(message, REPLY_TERMFREQ);
-    const char * p = message.data();
-    const char * p_end = p + message.size();
-    return decode_length(&p, p_end, false);
+    const char * p;
+    const char * p_end;
+    if (termfreq_ptr) {
+	if (collfreq_ptr) {
+	    send_message(MSG_FREQS, term);
+	    get_message(message, REPLY_FREQS);
+	} else {
+	    send_message(MSG_TERMFREQ, term);
+	    get_message(message, REPLY_TERMFREQ);
+	}
+	p = message.data();
+	p_end = p + message.size();
+	decode_length(&p, p_end, *termfreq_ptr);
+    } else if (collfreq_ptr) {
+	send_message(MSG_COLLFREQ, term);
+	get_message(message, REPLY_COLLFREQ);
+	p = message.data();
+	p_end = p + message.size();
+    }
+    if (collfreq_ptr) {
+	decode_length(&p, p_end, *collfreq_ptr);
+    }
 }
 
-Xapian::termcount
-RemoteDatabase::get_collection_freq(const string & tname) const
-{
-    Assert(!tname.empty());
-    send_message(MSG_COLLFREQ, tname);
-    string message;
-    get_message(message, REPLY_COLLFREQ);
-    const char * p = message.data();
-    const char * p_end = p + message.size();
-    return decode_length(&p, p_end, false);
-}
-
-
 void
 RemoteDatabase::read_value_stats(Xapian::valueno slot) const
 {
@@ -433,11 +463,12 @@ RemoteDatabase::read_value_stats(Xapian::valueno slot) const
 	const char * p = message.data();
 	const char * p_end = p + message.size();
 	mru_slot = slot;
-	mru_valstats.freq = decode_length(&p, p_end, false);
-	size_t len = decode_length(&p, p_end, true);
+	decode_length(&p, p_end, mru_valstats.freq);
+	size_t len;
+	decode_length_and_check(&p, p_end, len);
 	mru_valstats.lower_bound.assign(p, len);
 	p += len;
-	len = decode_length(&p, p_end, true);
+	decode_length_and_check(&p, p_end, len);
 	mru_valstats.upper_bound.assign(p, len);
 	p += len;
 	if (p != p_end) {
@@ -498,18 +529,39 @@ RemoteDatabase::get_doclength(Xapian::docid did) const
     get_message(message, REPLY_DOCLENGTH);
     const char * p = message.c_str();
     const char * p_end = p + message.size();
-    Xapian::termcount doclen = decode_length(&p, p_end, false);
+    Xapian::termcount doclen;
+    decode_length(&p, p_end, doclen);
     if (p != p_end) {
 	throw Xapian::NetworkError("Bad REPLY_DOCLENGTH message received", context);
     }
     return doclen;
 }
 
+Xapian::termcount
+RemoteDatabase::get_unique_terms(Xapian::docid did) const
+{
+    Assert(did != 0);
+    send_message(MSG_UNIQUETERMS, encode_length(did));
+    string message;
+    get_message(message, REPLY_UNIQUETERMS);
+    const char * p = message.c_str();
+    const char * p_end = p + message.size();
+    Xapian::termcount doclen;
+    decode_length(&p, p_end, doclen);
+    if (p != p_end) {
+	throw Xapian::NetworkError("Bad REPLY_UNIQUETERMS message received", context);
+    }
+    return doclen;
+}
+
 reply_type
 RemoteDatabase::get_message(string &result, reply_type required_type) const
 {
     double end_time = RealTime::end_time(timeout);
-    reply_type type = static_cast<reply_type>(link.get_message(result, end_time));
+    int type_int = link.get_message(result, end_time);
+    if (type_int < 0)
+	throw_connection_closed_unexpectedly();
+    reply_type type = static_cast<reply_type>(type_int);
     if (type == REPLY_EXCEPTION) {
 	unserialise_error(result, "REMOTE:", context);
     }
@@ -550,7 +602,7 @@ RemoteDatabase::do_close()
 }
 
 void
-RemoteDatabase::set_query(const Xapian::Query::Internal *query,
+RemoteDatabase::set_query(const Xapian::Query& query,
 			 Xapian::termcount qlen,
 			 Xapian::doccount collapse_max,
 			 Xapian::valueno collapse_key,
@@ -558,12 +610,13 @@ RemoteDatabase::set_query(const Xapian::Query::Internal *query,
 			 Xapian::valueno sort_key,
 			 Xapian::Enquire::Internal::sort_setting sort_by,
 			 bool sort_value_forward,
-			 int percent_cutoff, Xapian::weight weight_cutoff,
+			 double time_limit,
+			 int percent_cutoff, double weight_cutoff,
 			 const Xapian::Weight *wtscheme,
 			 const Xapian::RSet &omrset,
-			 const vector<Xapian::MatchSpy *> & matchspies)
+			 const vector<Xapian::Internal::opt_intrusive_ptr<Xapian::MatchSpy>> & matchspies)
 {
-    string tmp = query->serialise();
+    string tmp = query.serialise();
     string message = encode_length(tmp.size());
     message += tmp;
 
@@ -575,6 +628,7 @@ RemoteDatabase::set_query(const Xapian::Query::Internal *query,
     message += encode_length(sort_key);
     message += char('0' + sort_by);
     message += char('0' + sort_value_forward);
+    message += serialise_double(time_limit);
     message += char(percent_cutoff);
     message += serialise_double(weight_cutoff);
 
@@ -590,21 +644,20 @@ RemoteDatabase::set_query(const Xapian::Query::Internal *query,
     message += encode_length(tmp.size());
     message += tmp;
 
-    vector<Xapian::MatchSpy *>::const_iterator i;
-    for (i = matchspies.begin(); i != matchspies.end(); ++i) {
-	tmp = (*i)->name();
+    for (auto i : matchspies) {
+	tmp = i->name();
 	if (tmp.empty()) {
 	    throw Xapian::UnimplementedError("MatchSpy subclass not suitable for use with remote searches - name() method returned empty string");
 	}
 	message += encode_length(tmp.size());
 	message += tmp;
 
-	tmp = (*i)->serialise();
+	tmp = i->serialise();
 	message += encode_length(tmp.size());
 	message += tmp;
     }
 
-    send_message(MSG_QUERY_NEW, message);
+    send_message(MSG_QUERY, message);
 }
 
 bool
@@ -614,7 +667,7 @@ RemoteDatabase::get_remote_stats(bool nowait, Xapian::Weight::Internal &out)
 
     string message;
     get_message(message, REPLY_STATS);
-    out = unserialise_stats(message);
+    unserialise_stats(message, out);
 
     return true;
 }
@@ -634,23 +687,23 @@ RemoteDatabase::send_global_stats(Xapian::doccount first,
 
 void
 RemoteDatabase::get_mset(Xapian::MSet &mset,
-			 const vector<Xapian::MatchSpy *> & matchspies)
+			 const vector<Xapian::Internal::opt_intrusive_ptr<Xapian::MatchSpy>> & matchspies)
 {
     string message;
-    get_message(message, REPLY_RESULTS_NEW);
+    get_message(message, REPLY_RESULTS);
     const char * p = message.data();
     const char * p_end = p + message.size();
 
-    vector<Xapian::MatchSpy *>::const_iterator i;
-    for (i = matchspies.begin(); i != matchspies.end(); ++i) {
+    for (auto i : matchspies) {
 	if (p == p_end)
 	    throw Xapian::NetworkError("Expected serialised matchspy");
-	size_t len = decode_length(&p, p_end, true);
+	size_t len;
+	decode_length_and_check(&p, p_end, len);
 	string spyresults(p, len);
 	p += len;
-	(*i)->merge_results(spyresults);
+	i->merge_results(spyresults);
     }
-    mset = unserialise_mset_new(p, p_end);
+    mset = unserialise_mset(p, p_end);
 }
 
 void
@@ -685,7 +738,9 @@ RemoteDatabase::add_document(const Xapian::Document & doc)
 
     const char * p = message.data();
     const char * p_end = p + message.size();
-    return decode_length(&p, p_end, false);
+    Xapian::docid did;
+    decode_length(&p, p_end, did);
+    return did;
 }
 
 void
@@ -738,7 +793,9 @@ RemoteDatabase::replace_document(const std::string & unique_term,
 
     const char * p = message.data();
     const char * p_end = p + message.size();
-    return decode_length(&p, p_end, false);
+    Xapian::docid did;
+    decode_length(&p, p_end, did);
+    return did;
 }
 
 string
