@@ -1,7 +1,7 @@
 /** @file esetinternal.cc
  * @brief Xapian::ESet::Internal class
  */
-/* Copyright (C) 2008,2010 Olly Betts
+/* Copyright (C) 2008,2010,2011,2013,2016 Olly Betts
  * Copyright (C) 2011 Action Without Borders
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,14 +25,15 @@
 
 #include "xapian/enquire.h"
 #include "xapian/expanddecider.h"
-#include "database.h"
+#include "backends/database.h"
 #include "debuglog.h"
-#include "omenquireinternal.h"
+#include "api/omenquireinternal.h"
 #include "expandweight.h"
 #include "omassert.h"
 #include "ortermlist.h"
 #include "str.h"
-#include "termlist.h"
+#include "api/termlist.h"
+#include "unicode/description_append.h"
 
 #include "autoptr.h"
 #include <set>
@@ -49,7 +50,7 @@ Internal::ExpandTerm::get_description() const
     string desc("ExpandTerm(");
     desc += str(wt);
     desc += ", ";
-    desc += term;
+    description_append(desc, term);
     desc += ')';
     return desc;
 }
@@ -139,8 +140,8 @@ ESet::Internal::expand(Xapian::termcount max_esize,
 		       const Xapian::Database & db,
 		       const RSet & rset,
 		       const Xapian::ExpandDecider * edecider,
-		       const Xapian::Internal::ExpandWeight & eweight,
-		       Xapian::weight min_wt)
+		       Xapian::Internal::ExpandWeight & eweight,
+		       double min_wt)
 {
     LOGCALL_VOID(EXPAND, "ESet::Internal::expand", max_esize | db | rset | edecider | eweight);
     // These two cases are handled by our caller.
@@ -172,7 +173,12 @@ ESet::Internal::expand(Xapian::termcount max_esize,
 
 	++ebound;
 
-	Xapian::weight wt = eweight.get_weight(tree.get(), term);
+	/* Set up the ExpandWeight by clearing the existing statistics and
+	   collecting statistics for the new term. */
+	eweight.collect_stats(tree.get(), term);
+
+	double wt = eweight.get_weight();
+
 	// If the weights are equal, we prefer the lexically smaller term and
 	// so we use "<=" not "<" here.
 	if (wt <= min_wt) continue;
@@ -218,4 +224,64 @@ ESet::Internal::get_description() const
     return desc;
 }
 
+ESet::ESet() { }
+
+ESet::ESet(const ESet & o) : internal(o.internal) { }
+
+ESet&
+ESet::operator=(const ESet & o)
+{
+    internal = o.internal;
+    return *this;
+}
+
+ESet::~ESet() { }
+
+Xapian::doccount
+ESet::size() const
+{
+    if (!internal.get()) return 0;
+    return internal->items.size();
+}
+
+Xapian::termcount
+ESet::get_ebound() const
+{
+    if (!internal.get()) return 0;
+    return internal->ebound;
+}
+
+std::string
+ESet::get_description() const
+{
+    string desc = "ESet(";
+    if (internal.get())
+	desc += internal->get_description();
+    desc += ')';
+    return desc;
+}
+
+
+std::string
+ESetIterator::operator*() const
+{
+    return (eset.internal->items.end() - off_from_end)->term;
+}
+
+double
+ESetIterator::get_weight() const
+{
+    return (eset.internal->items.end() - off_from_end)->wt;
+}
+
+std::string
+ESetIterator::get_description() const
+{
+    string desc = "ESetIterator(";
+    if (eset.internal.get())
+	desc += str(eset.internal->items.size() - off_from_end);
+    desc += ')';
+    return desc;
+}
+
 }
