@@ -5,7 +5,7 @@ const char * dummy[] = {
 "/** @file version.h",
 " * @brief Define preprocessor symbols for the library version",
 " */",
-"// Copyright (C) 2002,2004,2005,2006,2007,2008,2009,2010,2015 Olly Betts",
+"// Copyright (C) 2002,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2015,2016 Olly Betts",
 "//",
 "// This program is free software; you can redistribute it and/or",
 "// modify it under the terms of the GNU General Public License as",
@@ -23,6 +23,13 @@ const char * dummy[] = {
 "",
 "#ifndef XAPIAN_INCLUDED_VERSION_H",
 "#define XAPIAN_INCLUDED_VERSION_H",
+"",
+// Disabled for now, since str.h is used by omega, and includes visibility.h
+// which includes version.h.  (FIXME)
+//"#if !defined XAPIAN_INCLUDED_XAPIAN_H && !defined XAPIAN_LIB_BUILD",
+//"# error @@Never use <xapian/version.h> directly; include <xapian.h> instead.@@",
+//"#endif",
+//"",
 #ifdef __GNUC__
 // When building the library with GCC, generate preprocessor code to check that
 // any version of GCC used to build applications has a matching C++ ABI. This
@@ -48,7 +55,6 @@ const char * dummy[] = {
 #define J2(A,B) g++ A##.##B
 #define V3(A,B,C) J3(A,B,C)
 #define J3(A,B,C) g++ A##.##B##.##C
-"",
 "#ifdef __GNUC__",
 "#if __GNUC__ < 3 || (__GNUC__ == 3 && __GNUC_MINOR__ == 0)",
 "#error Xapian no longer supports GCC < 3.1",
@@ -107,6 +113,26 @@ const char * dummy[] = {
 "#endif",
 "#endif",
 "",
+#elif defined _MSC_VER
+// When building the library with MSVC, generate preprocessor code to check
+// that the same setting of _DEBUG is used for building applications as was
+// used for building the library.
+"#ifdef _MSC_VER",
+#ifdef _DEBUG
+"#ifndef _DEBUG",
+"#error This library was compiled with _DEBUG defined, but you",
+"#error have not specified this flag.  The settings must match or your",
+"#error program will not work correctly.",
+"#endif",
+#else
+"#ifdef _DEBUG",
+"#error You are compiling with _DEBUG defined, but the library",
+"#error was not compiled with this flag.  The settings must match or your",
+"#error program will not work correctly.",
+"#endif",
+#endif
+"#endif",
+"",
 #endif
 #ifdef XAPIAN_ENABLE_VISIBILITY
 "/// The library was compiled with GCC's -fvisibility=hidden option.",
@@ -131,12 +157,14 @@ const char * dummy[] = {
 " */",
 "#define XAPIAN_REVISION ",REVISION,
 "",
-"/// XAPIAN_HAS_BRASS_BACKEND Defined if the brass backend is enabled.",
-#ifdef XAPIAN_HAS_BRASS_BACKEND
-"#define XAPIAN_HAS_BRASS_BACKEND 1",
-#else
-"/* #undef XAPIAN_HAS_BRASS_BACKEND */",
-#endif
+"/// Base (signed) type for Xapian::docid and related types.",
+"#define XAPIAN_DOCID_BASE_TYPE ",XAPIAN_DOCID_BASE_TYPE,
+"",
+"/// Base (signed) type for Xapian::termcount and related types.",
+"#define XAPIAN_TERMCOUNT_BASE_TYPE ",XAPIAN_TERMCOUNT_BASE_TYPE,
+"",
+"/// Underlying type for Xapian::rev.",
+"#define XAPIAN_REVISION_TYPE ",XAPIAN_REVISION_TYPE,
 "",
 "/// XAPIAN_HAS_CHERT_BACKEND Defined if the chert backend is enabled.",
 #ifdef XAPIAN_HAS_CHERT_BACKEND
@@ -145,11 +173,11 @@ const char * dummy[] = {
 "/* #undef XAPIAN_HAS_CHERT_BACKEND */",
 #endif
 "",
-"/// XAPIAN_HAS_FLINT_BACKEND Defined if the flint backend is enabled.",
-#ifdef XAPIAN_HAS_FLINT_BACKEND
-"#define XAPIAN_HAS_FLINT_BACKEND 1",
+"/// XAPIAN_HAS_GLASS_BACKEND Defined if the glass backend is enabled.",
+#ifdef XAPIAN_HAS_GLASS_BACKEND
+"#define XAPIAN_HAS_GLASS_BACKEND 1",
 #else
-"/* #undef XAPIAN_HAS_FLINT_BACKEND */",
+"/* #undef XAPIAN_HAS_GLASS_BACKEND */",
 #endif
 "",
 "/// XAPIAN_HAS_INMEMORY_BACKEND Defined if the inmemory backend is enabled.",
