@@ -229,17 +229,17 @@ DEFINE_TESTCASE(modifiedpostlist1, writable) {
     Xapian::WritableDatabase db = get_writable_database();
     Xapian::Document a, b;
     Xapian::Enquire enq(db);
-   
+
     a.add_term("T");
     enq.set_query(Xapian::Query("T"));
-   
+
     db.replace_document(2, a);
     db.commit();
     db.replace_document(1, a);
     db.replace_document(1, b);
-   
+
     mset_expect_order(enq.get_mset(0, 2), 2);
-   
+
     return true;
 }
 
@@ -331,7 +331,7 @@ DEFINE_TESTCASE(lockfilealreadyopen1, chert || glass) {
 
 struct MyMatchDecider : public Xapian::MatchDecider {
     mutable bool called;
-  
+
     MyMatchDecider() : called(false) { }
 
     bool operator()(const Xapian::Document &) const {
@@ -479,7 +479,7 @@ DEFINE_TESTCASE(qpmemoryleak1, writable && !inmemory) {
     }
 
     for (int j = 0; j < 50; ++j) {
-        wdb.add_document(doc);
+	wdb.add_document(doc);
     }
     wdb.commit();
 
@@ -1433,3 +1433,23 @@ DEFINE_TESTCASE(exactxor1, backend) {
 
     return true;
 }
+
+/// Feature test for Database::get_revision().
+DEFINE_TESTCASE(getrevision1, chert || glass) {
+    Xapian::WritableDatabase db = get_writable_database();
+    TEST_EQUAL(db.get_revision(), 0);
+    db.commit();
+    TEST_EQUAL(db.get_revision(), 0);
+    Xapian::Document doc;
+    doc.add_term("hello");
+    db.add_document(doc);
+    TEST_EQUAL(db.get_revision(), 0);
+    db.commit();
+    TEST_EQUAL(db.get_revision(), 1);
+    db.commit();
+    TEST_EQUAL(db.get_revision(), 1);
+    db.add_document(doc);
+    db.commit();
+    TEST_EQUAL(db.get_revision(), 2);
+    return true;
+}
