@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2012,2013 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2011,2012,2013,2015,2016 Olly Betts
  * Copyright 2006,2007,2008,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -30,6 +30,7 @@
 #include <map>
 #include <string>
 #include <vector>
+#include "safenetdb.h" // For gai_strerror().
 #include "safesysstat.h" // For mkdir().
 #include "safeunistd.h" // For sleep().
 
@@ -40,7 +41,6 @@
 #include "testsuite.h"
 #include "testutils.h"
 #include "unixcmds.h"
-#include "utils.h"
 
 #include "apitest.h"
 
@@ -80,7 +80,7 @@ DEFINE_TESTCASE(stubdb1, backend && !inmemory && !remote) {
     out.close();
 
     {
-	Xapian::Database db = Xapian::Auto::open_stub(dbpath);
+	Xapian::Database db(dbpath, Xapian::DB_BACKEND_STUB);
 	Xapian::Enquire enquire(db);
 	enquire.set_query(Xapian::Query("word"));
 	enquire.get_mset(0, 10);
@@ -107,7 +107,7 @@ DEFINE_TESTCASE(stubdb2, backend && !inmemory && !remote) {
     out.close();
 
     {
-	Xapian::Database db = Xapian::Auto::open_stub(dbpath);
+	Xapian::Database db(dbpath, Xapian::DB_BACKEND_STUB);
 	Xapian::Enquire enquire(db);
 	enquire.set_query(Xapian::Query("word"));
 	enquire.get_mset(0, 10);
@@ -119,6 +119,85 @@ DEFINE_TESTCASE(stubdb2, backend && !inmemory && !remote) {
 	enquire.get_mset(0, 10);
     }
 
+    out.open(dbpath);
+    TEST(out.is_open());
+    out << "remote" << endl;
+    out.close();
+
+    // Quietly ignored prior to 1.4.1.
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::Database db(dbpath, Xapian::DB_BACKEND_STUB)
+    );
+
+    // Quietly ignored prior to 1.4.1.
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::WritableDatabase db(dbpath, Xapian::DB_BACKEND_STUB)
+    );
+
+    out.open(dbpath);
+    TEST(out.is_open());
+    out << "remote foo" << endl;
+    out.close();
+
+    // Quietly ignored prior to 1.4.1.
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::Database db(dbpath, Xapian::DB_BACKEND_STUB)
+    );
+
+    // Quietly ignored prior to 1.4.1.
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::WritableDatabase db(dbpath, Xapian::DB_BACKEND_STUB)
+    );
+
+    out.open(dbpath);
+    TEST(out.is_open());
+    out << "remote [::1]:80" << endl;
+    out.close();
+
+    try {
+	Xapian::Database db(dbpath, Xapian::DB_BACKEND_STUB);
+    } catch (const Xapian::NetworkError& e) {
+	// 1.4.0 threw (Linux):
+	//  NetworkError: Couldn't resolve host [ (context: remote:tcp([:0)) (No address associated with hostname)
+	// 1.4.1 throws (because we don't actually support IPv6 yet) on Linux (EAI_ADDRFAMILY):
+	//  NetworkError: Couldn't resolve host ::1 (context: remote:tcp(::1:80)) (nodename nor servname provided, or not known)
+	// or on OS X (EAI_NONAME):
+	//  NetworkError: Couldn't resolve host ::1 (context: remote:tcp(::1:80)) (Address family for hostname not supported)
+	// So we test the message instead of the error string for portability.
+	TEST(e.get_msg().find("host ::1") != string::npos);
+    }
+
+    try {
+	Xapian::WritableDatabase db(dbpath, Xapian::DB_BACKEND_STUB);
+    } catch (const Xapian::NetworkError& e) {
+	// 1.4.0 threw (Linux):
+	//  NetworkError: Couldn't resolve host [ (context: remote:tcp([:0)) (No address associated with hostname)
+	// 1.4.1 throws (because we don't actually support IPv6 yet) on Linux (EAI_ADDRFAMILY):
+	//  NetworkError: Couldn't resolve host ::1 (context: remote:tcp(::1:80)) (nodename nor servname provided, or not known)
+	// or on OS X (EAI_NONAME):
+	//  NetworkError: Couldn't resolve host ::1 (context: remote:tcp(::1:80)) (Address family for hostname not supported)
+	// So we test the message instead of the error string for portability.
+	TEST(e.get_msg().find("host ::1") != string::npos);
+    }
+
+    out.open(dbpath);
+    TEST(out.is_open());
+    // Invalid - the port number is required.
+    out << "remote [::1]" << endl;
+    out.close();
+
+    // 1.4.0 threw:
+    // NetworkError: Couldn't resolve host [ (context: remote:tcp([:0)) (No address associated with hostname)
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::Database db(dbpath, Xapian::DB_BACKEND_STUB);
+    );
+
+    // 1.4.0 threw:
+    // NetworkError: Couldn't resolve host [ (context: remote:tcp([:0)) (No address associated with hostname)
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::WritableDatabase db(dbpath, Xapian::DB_BACKEND_STUB);
+    );
+
     return true;
 }
 
@@ -134,7 +213,7 @@ DEFINE_TESTCASE(stubdb3, backend && !inmemory && !remote) {
     out.close();
 
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-	Xapian::Database db = Xapian::Auto::open_stub(dbpath));
+	Xapian::Database db(dbpath, Xapian::DB_BACKEND_STUB));
 
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
 	Xapian::Database db(dbpath));
@@ -153,7 +232,7 @@ DEFINE_TESTCASE(stubdb4, backend && !inmemory && !remote) {
     out.close();
 
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-	Xapian::Database db = Xapian::Auto::open_stub(dbpath));
+	Xapian::Database db(dbpath, Xapian::DB_BACKEND_STUB));
 
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
 	Xapian::Database db(dbpath));
@@ -174,7 +253,7 @@ DEFINE_TESTCASE(stubdb5, backend && !inmemory && !remote) {
     out.close();
 
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-	Xapian::Database db = Xapian::Auto::open_stub(dbpath));
+	Xapian::Database db(dbpath, Xapian::DB_BACKEND_STUB));
 
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
 	Xapian::Database db(dbpath));
@@ -193,7 +272,7 @@ DEFINE_TESTCASE(stubdb6, inmemory) {
 
     // Read-only tests:
     {
-	Xapian::Database db = Xapian::Auto::open_stub(dbpath);
+	Xapian::Database db(dbpath, Xapian::DB_BACKEND_STUB);
 	TEST_EQUAL(db.get_doccount(), 0);
 	Xapian::Enquire enquire(db);
 	enquire.set_query(Xapian::Query("word"));
@@ -211,19 +290,20 @@ DEFINE_TESTCASE(stubdb6, inmemory) {
 
     // Writable tests:
     {
-	Xapian::WritableDatabase db;
-        db = Xapian::Auto::open_stub(dbpath, Xapian::DB_OPEN);
+	Xapian::WritableDatabase db(dbpath,
+		Xapian::DB_OPEN|Xapian::DB_BACKEND_STUB);
 	TEST_EQUAL(db.get_doccount(), 0);
 	db.add_document(Xapian::Document());
 	TEST_EQUAL(db.get_doccount(), 1);
     }
     {
-	Xapian::WritableDatabase db(dbpath, Xapian::DB_OPEN);
+	Xapian::WritableDatabase db(dbpath,
+		Xapian::DB_OPEN|Xapian::DB_BACKEND_STUB);
 	TEST_EQUAL(db.get_doccount(), 0);
 	db.add_document(Xapian::Document());
 	TEST_EQUAL(db.get_doccount(), 1);
     }
- 
+
     return true;
 }
 
@@ -243,7 +323,7 @@ class MyErrorHandler : public Xapian::ErrorHandler {
 };
 
 // tests error handler in multimatch().
-//DEFINE_TESTCASE(multierrhandler1, backend) {
+DEFINE_TESTCASE(multierrhandler1, backend) {
     MyErrorHandler myhandler;
 
     Xapian::Database mydb2(get_database("apitest_simpledata"));
@@ -915,11 +995,11 @@ DEFINE_TESTCASE(allterms1, backend) {
     ati++;
     TEST(ati != db.allterms_end());
     if (verbose) {
-	tout << "*ati = `" << *ati << "'\n";
-	tout << "*ati.length = `" << (*ati).length() << "'\n";
+	tout << "*ati = '" << *ati << "'\n";
+	tout << "*ati.length = '" << (*ati).length() << "'\n";
 	tout << "*ati == \"one\" = " << (*ati == "one") << "\n";
 	tout << "*ati[3] = " << ((*ati)[3]) << "\n";
-	tout << "*ati = `" << *ati << "'\n";
+	tout << "*ati = '" << *ati << "'\n";
     }
     TEST(*ati == "three");
     TEST(ati.get_termfreq() == 3);
@@ -1265,19 +1345,19 @@ DEFINE_TESTCASE(postlist2, backend) {
     p = db.postlist_begin("this");
     Xapian::PostingIterator pend = db.postlist_end("this");
 
-    TEST(p.get_description() != "Xapian::PostingIterator(pos=END)");
+    TEST(p.get_description() != "PostingIterator()");
 
     // test operator= creates a copy which compares equal
     Xapian::PostingIterator p_copy = p;
     TEST_EQUAL(p, p_copy);
 
-    TEST(p_copy.get_description() != "Xapian::PostingIterator(pos=END)");
+    TEST(p_copy.get_description() != "PostingIterator()");
 
     // test copy constructor creates a copy which compares equal
     Xapian::PostingIterator p_clone(p);
     TEST_EQUAL(p, p_clone);
 
-    TEST(p_clone.get_description() != "Xapian::PostingIterator(pos=END)");
+    TEST(p_clone.get_description() != "PostingIterator()");
 
     vector<Xapian::docid> v(p, pend);
 
@@ -1291,10 +1371,8 @@ DEFINE_TESTCASE(postlist2, backend) {
     }
     TEST_EQUAL(p, pend);
 
-    TEST_STRINGS_EQUAL(p.get_description(),
-		       "Xapian::PostingIterator(pos=END)");
-    TEST_STRINGS_EQUAL(pend.get_description(),
-		       "Xapian::PostingIterator(pos=END)");
+    TEST_STRINGS_EQUAL(p.get_description(), "PostingIterator()");
+    TEST_STRINGS_EQUAL(pend.get_description(), "PostingIterator()");
 
     return true;
 }
@@ -1353,7 +1431,13 @@ DEFINE_TESTCASE(postlist6, backend) {
     TEST(i != db.postlist_end("this"));
     while (i != db.postlist_end("this")) {
 	TEST_EQUAL(i.get_doclength(), db.get_doclength(*i));
+	TEST_EQUAL(i.get_unique_terms(), db.get_unique_terms(*i));
 	TEST_REL(i.get_wdf(),<=,i.get_doclength());
+	TEST_REL(1,<=,i.get_unique_terms());
+	// The next two aren't necessarily true if there are terms with wdf=0
+	// in the document, but that isn't the case here.
+	TEST_REL(i.get_unique_terms(),<=,i.get_doclength());
+	TEST_REL(i.get_wdf() + i.get_unique_terms() - 1,<=,i.get_doclength());
 	++i;
     }
     return true;
@@ -1465,213 +1549,98 @@ DEFINE_TESTCASE(consistency1, backend && !remote) {
 }
 
 // tests that specifying a nonexistent input file throws an exception.
-DEFINE_TESTCASE(flintdatabaseopeningerror1, flint) {
-#ifdef XAPIAN_HAS_FLINT_BACKEND
-    mkdir(".flint", 0755);
+DEFINE_TESTCASE(chertdatabaseopeningerror1, chert) {
+#ifdef XAPIAN_HAS_CHERT_BACKEND
+    mkdir(".chert", 0755);
 
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   Xapian::Flint::open(".flint/nosuchdirectory"));
+	    Xapian::Database(".chert/nosuchdirectory",
+		Xapian::DB_BACKEND_CHERT));
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   Xapian::Flint::open(".flint/nosuchdirectory", Xapian::DB_OPEN));
+	    Xapian::WritableDatabase(".chert/nosuchdirectory",
+		Xapian::DB_OPEN|Xapian::DB_BACKEND_CHERT));
 
-    mkdir(".flint/emptydirectory", 0700);
+    mkdir(".chert/emptydirectory", 0700);
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   Xapian::Flint::open(".flint/emptydirectory"));
+	    Xapian::Database(".chert/emptydirectory",
+		Xapian::DB_BACKEND_CHERT));
 
-    touch(".flint/somefile");
+    touch(".chert/somefile");
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   Xapian::Flint::open(".flint/somefile"));
+	    Xapian::Database(".chert/somefile",
+		Xapian::DB_BACKEND_CHERT));
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   Xapian::Flint::open(".flint/somefile", Xapian::DB_OPEN));
+	    Xapian::WritableDatabase(".chert/somefile",
+		Xapian::DB_OPEN|Xapian::DB_BACKEND_CHERT));
     TEST_EXCEPTION(Xapian::DatabaseCreateError,
-		   Xapian::Flint::open(".flint/somefile", Xapian::DB_CREATE));
+	    Xapian::WritableDatabase(".chert/somefile",
+		Xapian::DB_CREATE|Xapian::DB_BACKEND_CHERT));
     TEST_EXCEPTION(Xapian::DatabaseCreateError,
-		   Xapian::Flint::open(".flint/somefile", Xapian::DB_CREATE_OR_OPEN));
+	    Xapian::WritableDatabase(".chert/somefile",
+		Xapian::DB_CREATE_OR_OPEN|Xapian::DB_BACKEND_CHERT));
     TEST_EXCEPTION(Xapian::DatabaseCreateError,
-		   Xapian::Flint::open(".flint/somefile", Xapian::DB_CREATE_OR_OVERWRITE));
+	    Xapian::WritableDatabase(".chert/somefile",
+		Xapian::DB_CREATE_OR_OVERWRITE|Xapian::DB_BACKEND_CHERT));
 #endif
 
     return true;
 }
 
-/// Tests that appropriate error is thrown for database format change.
-DEFINE_TESTCASE(flintdatabaseformaterror1, flint) {
-#ifdef XAPIAN_HAS_FLINT_BACKEND
-    string dbdir = test_driver::get_srcdir();
-    dbdir += "/testdata/flint-0.9.9";
-
-    // We should get a version error when we try and open the database for
-    // reading now.
-    TEST_EXCEPTION(Xapian::DatabaseVersionError,
-		   Xapian::Database db(dbdir));
-
-    // Also test when explicitly opening as a flint database.
-    TEST_EXCEPTION(Xapian::DatabaseVersionError,
-		   (void)Xapian::Flint::open(dbdir));
-
-    // Clean up the "flintlock" file that we will have created while trying
-    // to open the database.
-    unlink((dbdir + "/flintlock").c_str());
-#endif
-
-    return true;
-}
-
-/// Test that an old database can be successfully overwritten when using
-// Xapian::DB_CREATE_OR_OVERWRITE.
-DEFINE_TESTCASE(flintdatabaseformaterror2, flint) {
-#ifdef XAPIAN_HAS_FLINT_BACKEND
-    string flint099 = test_driver::get_srcdir();
-    flint099 += "/testdata/flint-0.9.9";
-
-    mkdir(".flint", 0755);
-    string dbdir = ".flint/test_flintdatabaseformaterror2";
-
-    rm_rf(dbdir);
-    cp_R(flint099, dbdir);
-
-    (void)Xapian::WritableDatabase(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
-
-    rm_rf(dbdir);
-    cp_R(flint099, dbdir);
-
-    // Also test when explicitly opening as a flint database.
-    (void)Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
-#endif
-
-    return true;
-}
-
-// regression test for not releasing lock on error.
-DEFINE_TESTCASE(flintdatabaseformaterror3, flint) {
-#ifdef XAPIAN_HAS_FLINT_BACKEND
-    string flint099 = test_driver::get_srcdir();
-    flint099 += "/testdata/flint-0.9.9";
-
-    mkdir(".flint", 0755);
-    string dbdir = ".flint/test_flintdatabaseformaterror3";
-
-    rm_rf(dbdir);
-    cp_R(flint099, dbdir);
-
-    TEST_EXCEPTION(Xapian::DatabaseVersionError,
-		   Xapian::WritableDatabase(dbdir, Xapian::DB_CREATE_OR_OPEN));
-
-    // This used to throw a DatabaseLockError: "Unable to acquire database
-    // write lock on .flint/formatdb: already locked"
-    Xapian::WritableDatabase(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
-#endif
-
-    return true;
-}
-
-// Test that 1.0.2 and later can open 1.0.1 databases.
-DEFINE_TESTCASE(flintbackwardcompat1, flint) {
-#ifdef XAPIAN_HAS_FLINT_BACKEND
-    string flint101 = test_driver::get_srcdir();
-    flint101 += "/testdata/flint-1.0.1";
-
-    mkdir(".flint", 0755);
-    string dbdir = ".flint/test_flintbackwardcompat1";
-
-    rm_rf(dbdir);
-    cp_R(flint101, dbdir);
-
-    // Check we can open the older format for reading.
-    {
-	Xapian::Database db(dbdir);
-	TEST_EQUAL(db.get_doccount(), 0);
-    }
-
-    // Check we can open the older format for update.
-    {
-	Xapian::WritableDatabase db(dbdir, Xapian::DB_OPEN);
-	TEST_EQUAL(db.get_doccount(), 0);
-    }
-#endif
-
-    return true;
-}
-
-// Test that 1.0.3 and later can open 1.0.2 databases.
-DEFINE_TESTCASE(flintbackwardcompat2, flint) {
-#ifdef XAPIAN_HAS_FLINT_BACKEND
-    string flint102 = test_driver::get_srcdir();
-    flint102 += "/testdata/flint-1.0.2";
-
-    mkdir(".flint", 0755);
-    string dbdir = ".flint/test_flintbackwardcompat2";
-
-    rm_rf(dbdir);
-    cp_R(flint102, dbdir);
-
-    // Check we can open the older format for reading.
-    {
-	Xapian::Database db(dbdir);
-	TEST_EQUAL(db.get_doccount(), 0);
-    }
-
-    // Check we can open the older format for update.
-    {
-	Xapian::WritableDatabase db(dbdir, Xapian::DB_OPEN);
-	TEST_EQUAL(db.get_doccount(), 0);
-    }
-#endif
-
-    return true;
-}
-
-/// Test opening of a flint database
-DEFINE_TESTCASE(flintdatabaseopen1, flint) {
-#ifdef XAPIAN_HAS_FLINT_BACKEND
-    const string dbdir = ".flint/test_flintdatabaseopen1";
-    mkdir(".flint", 0755);
+/// Test opening of a chert database
+DEFINE_TESTCASE(chertdatabaseopen1, chert) {
+#ifdef XAPIAN_HAS_CHERT_BACKEND
+    const string dbdir = ".chert/test_chertdatabaseopen1";
+    mkdir(".chert", 0755);
 
     {
 	rm_rf(dbdir);
-	Xapian::WritableDatabase wdb =
-	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE);
+	Xapian::WritableDatabase wdb(dbdir,
+		Xapian::DB_CREATE|Xapian::DB_BACKEND_CHERT);
 	TEST_EXCEPTION(Xapian::DatabaseLockError,
-	    Xapian::Flint::open(dbdir, Xapian::DB_OPEN));
-	Xapian::Flint::open(dbdir);
+	    Xapian::WritableDatabase(dbdir,
+		Xapian::DB_OPEN|Xapian::DB_BACKEND_CHERT));
+	Xapian::Database(dbdir, Xapian::DB_BACKEND_CHERT);
     }
 
     {
 	rm_rf(dbdir);
-	Xapian::WritableDatabase wdb =
-	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OPEN);
+	Xapian::WritableDatabase wdb(dbdir,
+		Xapian::DB_CREATE_OR_OPEN|Xapian::DB_BACKEND_CHERT);
 	TEST_EXCEPTION(Xapian::DatabaseLockError,
-	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE));
-	Xapian::Flint::open(dbdir);
+	    Xapian::WritableDatabase(dbdir,
+		Xapian::DB_CREATE_OR_OVERWRITE|Xapian::DB_BACKEND_CHERT));
+	Xapian::Database(dbdir, Xapian::DB_BACKEND_CHERT);
     }
 
     {
 	rm_rf(dbdir);
-	Xapian::WritableDatabase wdb =
-	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
+	Xapian::WritableDatabase wdb(dbdir,
+		Xapian::DB_CREATE_OR_OVERWRITE|Xapian::DB_BACKEND_CHERT);
 	TEST_EXCEPTION(Xapian::DatabaseLockError,
-	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OPEN));
-	Xapian::Flint::open(dbdir);
+	    Xapian::WritableDatabase(dbdir,
+		Xapian::DB_CREATE_OR_OPEN|Xapian::DB_BACKEND_CHERT));
+	Xapian::Database(dbdir, Xapian::DB_BACKEND_CHERT);
     }
 
     {
 	TEST_EXCEPTION(Xapian::DatabaseCreateError,
-	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE));
-	Xapian::WritableDatabase wdb =
-	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
-	Xapian::Flint::open(dbdir);
+	    Xapian::WritableDatabase(dbdir,
+		Xapian::DB_CREATE|Xapian::DB_BACKEND_CHERT));
+	Xapian::WritableDatabase wdb(dbdir,
+		Xapian::DB_CREATE_OR_OVERWRITE|Xapian::DB_BACKEND_CHERT);
+	Xapian::Database(dbdir, Xapian::DB_BACKEND_CHERT);
     }
 
     {
-	Xapian::WritableDatabase wdb =
-	    Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OPEN);
-	Xapian::Flint::open(dbdir);
+	Xapian::WritableDatabase wdb(dbdir,
+		Xapian::DB_CREATE_OR_OPEN|Xapian::DB_BACKEND_CHERT);
+	Xapian::Database(dbdir, Xapian::DB_BACKEND_CHERT);
     }
 
     {
-	Xapian::WritableDatabase wdb =
-	    Xapian::Flint::open(dbdir, Xapian::DB_OPEN);
-	Xapian::Flint::open(dbdir);
+	Xapian::WritableDatabase wdb(dbdir,
+		Xapian::DB_OPEN|Xapian::DB_BACKEND_CHERT);
+	Xapian::Database(dbdir, Xapian::DB_BACKEND_CHERT);
     }
 #endif
 
@@ -1901,13 +1870,13 @@ class MyWeight : public Xapian::Weight {
     std::string name() const { return "MyWeight"; }
     string serialise() const { return string(); }
     MyWeight * unserialise(const string &) const { return new MyWeight; }
-    Xapian::weight get_sumpart(Xapian::termcount, Xapian::termcount) const {
+    double get_sumpart(Xapian::termcount, Xapian::termcount, Xapian::termcount) const {
 	return scale_factor;
     }
-    Xapian::weight get_maxpart() const { return scale_factor; }
+    double get_maxpart() const { return scale_factor; }
 
-    Xapian::weight get_sumextra(Xapian::termcount) const { return 0; }
-    Xapian::weight get_maxextra() const { return 0; }
+    double get_sumextra(Xapian::termcount, Xapian::termcount) const { return 0; }
+    double get_maxextra() const { return 0; }
 };
 
 // tests user weighting scheme.
@@ -1959,7 +1928,7 @@ DEFINE_TESTCASE(matchall1, backend) {
     TEST_EQUAL(mset[mset.size() - 1].get_weight(), 0);
     TEST_EQUAL(*mset[0], 1);
     TEST_EQUAL(*mset[mset.size() - 1], mset.size());
- 
+
     return true;
 }
 
