@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 Hein Ragas
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2014,2015 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2014,2015 Olly Betts
  * Copyright 2006 Richard Boulton
  * Copyright 2007 Lemur Consulting Ltd
  *
@@ -29,16 +29,17 @@
 
 #include <xapian.h>
 
-#include "backendmanager.h" // For XAPIAN_BIN_PATH.
+#include "filetests.h"
 #include "omassert.h"
 #include "str.h"
+#include "stringutils.h"
 #include "testsuite.h"
 #include "testutils.h"
 #include "unixcmds.h"
-#include "utils.h"
 
 #include "apitest.h"
 
+#include "safeunistd.h"
 #include <cmath>
 #include <cstdlib>
 #include <map>
@@ -286,9 +287,10 @@ DEFINE_TESTCASE(adddoc3, writable) {
     return true;
 }
 
-// We want to test that a termlist starting with a 48 character long term works
-// OK since this value currently requires special handling in flint for
-// historical reasons!)  Also test all other term lengths while we're at it.
+// We originally wanted to test that a termlist starting with a 48 character
+// long term worked since that required special handling in flint for
+// historical reasons.  That's no longer relevant, but it seems useful to
+// continue to test term lists starting with various term lengths work.
 DEFINE_TESTCASE(adddoc4, writable) {
     Xapian::WritableDatabase db = get_writable_database();
 
@@ -423,13 +425,10 @@ DEFINE_TESTCASE(adddoc5, writable) {
 		TEST_NOT_EQUAL(j, document_out.termlist_end());
 		TEST_EQUAL(*i, *j);
 		TEST_EQUAL(i.get_wdf(), j.get_wdf());
-#ifndef __clang__
-		// clang's exception handling is buggy, and this exception
-		// leaks through catches in the test harness.  Confirmed as a
-		// bug by a clang developer, but it seems no fix is in sight.
+		// Actually use termfreq to stop compiler optimising away the
+		// call to get_termfreq().
 		TEST_EXCEPTION(Xapian::InvalidOperationError,
-			       (void)i.get_termfreq());
-#endif
+			       if (i.get_termfreq()) return false);
 		TEST_NOT_EQUAL(0, j.get_termfreq());
 		if (*i == "foobar") {
 		    // termfreq of foobar is 2
@@ -607,7 +606,8 @@ DEFINE_TESTCASE(deldoc2, writable) {
 
     db.commit();
 
-    db.reopen();
+    // reopen() on a writable database shouldn't do anything.
+    TEST(!db.reopen());
 
     db.delete_document(1);
     db.delete_document(2);
@@ -615,7 +615,8 @@ DEFINE_TESTCASE(deldoc2, writable) {
 
     db.commit();
 
-    db.reopen();
+    // reopen() on a writable database shouldn't do anything.
+    TEST(!db.reopen());
 
     TEST_EQUAL(db.postlist_begin("one"), db.postlist_end("one"));
     TEST_EQUAL(db.postlist_begin("two"), db.postlist_end("two"));
@@ -646,6 +647,10 @@ DEFINE_TESTCASE(deldoc2, writable) {
     TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_doclength(2));
     TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_doclength(3));
 
+    TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_unique_terms(1));
+    TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_unique_terms(2));
+    TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_unique_terms(3));
+
     TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_document(1));
     TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_document(2));
     TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_document(3));
@@ -668,13 +673,15 @@ DEFINE_TESTCASE(deldoc3, writable) {
 
     db.commit();
 
-    db.reopen();
+    // reopen() on a writable database shouldn't do anything.
+    TEST(!db.reopen());
 
     db.delete_document(1);
 
     db.commit();
 
-    db.reopen();
+    // reopen() on a writable database shouldn't do anything.
+    TEST(!db.reopen());
 
     TEST_EQUAL(db.postlist_begin("one"), db.postlist_end("one"));
 
@@ -695,6 +702,9 @@ DEFINE_TESTCASE(deldoc3, writable) {
     TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_doclength(1));
     TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_doclength(2));
 
+    TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_unique_terms(1));
+    TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_unique_terms(2));
+
     TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_document(1));
     TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_document(2));
 
@@ -734,11 +744,13 @@ DEFINE_TESTCASE(deldoc4, writable) {
 	bool is_power_of_two = ((i & (i - 1)) == 0);
 	if (is_power_of_two) {
 	    db.commit();
-	    db.reopen();
+	    // reopen() on a writable database shouldn't do anything.
+	    TEST(!db.reopen());
 	}
     }
     db.commit();
-    db.reopen();
+    // reopen() on a writable database shouldn't do anything.
+    TEST(!db.reopen());
 
     /* delete the documents in a peculiar order */
     for (Xapian::docid i = 0; i < maxdoc / 3; ++i) {
@@ -748,8 +760,8 @@ DEFINE_TESTCASE(deldoc4, writable) {
     }
 
     db.commit();
-
-    db.reopen();
+    // reopen() on a writable database shouldn't do anything.
+    TEST(!db.reopen());
 
     TEST_EQUAL(db.postlist_begin("one"), db.postlist_end("one"));
     TEST_EQUAL(db.postlist_begin("two"), db.postlist_end("two"));
@@ -764,6 +776,7 @@ DEFINE_TESTCASE(deldoc4, writable) {
 	tout.str(string());
 	TEST_EXCEPTION(Xapian::DocNotFoundError, db.termlist_begin(i));
 	TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_doclength(i));
+	TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_unique_terms(i));
 	TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_document(i));
     }
 
@@ -1010,14 +1023,20 @@ DEFINE_TESTCASE(replacedoc3, writable) {
     TEST_EQUAL(db.get_doclength(2), 1);
     TEST_EQUAL(db.get_doclength(3), 4);
 
+    TEST_EQUAL(db.get_unique_terms(1), 3);
+    TEST_EQUAL(db.get_unique_terms(2), 1);
+    TEST_EQUAL(db.get_unique_terms(3), 4);
+
     Xapian::PostingIterator p = db.postlist_begin("foo");
     TEST_NOT_EQUAL(p, db.postlist_end("foo"));
     TEST_EQUAL(*p, 1);
     TEST_EQUAL(p.get_doclength(), 3);
+    TEST_EQUAL(p.get_unique_terms(), 3);
     ++p;
     TEST_NOT_EQUAL(p, db.postlist_end("foo"));
     TEST_EQUAL(*p, 3);
     TEST_EQUAL(p.get_doclength(), 4);
+    TEST_EQUAL(p.get_unique_terms(), 4);
     ++p;
     TEST_EQUAL(p, db.postlist_end("foo"));
 
@@ -1025,10 +1044,12 @@ DEFINE_TESTCASE(replacedoc3, writable) {
     TEST_NOT_EQUAL(p, db.postlist_end("world"));
     TEST_EQUAL(*p, 2);
     TEST_EQUAL(p.get_doclength(), 1);
+    TEST_EQUAL(p.get_unique_terms(), 1);
     ++p;
     TEST_NOT_EQUAL(p, db.postlist_end("world"));
     TEST_EQUAL(*p, 3);
     TEST_EQUAL(p.get_doclength(), 4);
+    TEST_EQUAL(p.get_unique_terms(), 4);
     ++p;
     TEST_EQUAL(p, db.postlist_end("world"));
 
@@ -1097,7 +1118,7 @@ DEFINE_TESTCASE(replacedoc4, writable) {
 
 // Test replacing a document with itself without modifying postings.
 // Regression test for bug in 0.9.9 and earlier - there flint and quartz
-// lose all positional information for the document when you do this.
+// lost all positional information for the document when you did this.
 DEFINE_TESTCASE(replacedoc5, writable) {
     Xapian::WritableDatabase db = get_writable_database();
 
@@ -1124,9 +1145,9 @@ DEFINE_TESTCASE(replacedoc5, writable) {
 	TEST(db.positionlist_begin(1, "world") != db.positionlist_end(1, "world"));
     }
 
-    // Brass, chert and flint now spot simple cases of replacing the same
-    // document and don't do needless work.  Force them to actually do the
-    // replacement to make sure that case works.
+    // The backends now spot simple cases of replacing the same document and
+    // don't do needless work.  Force them to actually do the replacement to
+    // make sure that case works.
 
     {
 	Xapian::Document doc;
@@ -1493,7 +1514,10 @@ DEFINE_TESTCASE(consistency2, writable) {
     return true;
 }
 
-DEFINE_TESTCASE(crashrecovery1, brass || chert || flint) {
+DEFINE_TESTCASE(crashrecovery1, chert) {
+    // Glass has a single version file per revision, rather than multiple base
+    // files, so it simply can't get into the situations we are testing
+    // recovery from.
     const string & dbtype = get_dbtype();
     string path = ".";
     path += dbtype;
@@ -1516,7 +1540,7 @@ DEFINE_TESTCASE(crashrecovery1, brass || chert || flint) {
 
 	db.add_document(doc);
 	db.commit();
-	dbr.reopen();
+	TEST(dbr.reopen());
 	TEST_EQUAL(dbr.get_doccount(), 1);
 
 	// Xapian::Database has full set of baseB, old baseA
@@ -1529,7 +1553,7 @@ DEFINE_TESTCASE(crashrecovery1, brass || chert || flint) {
 
 	db.add_document(doc);
 	db.commit();
-	dbr.reopen();
+	TEST(dbr.reopen());
 	TEST_EQUAL(dbr.get_doccount(), 2);
 
 	// Xapian::Database has full set of baseA, old baseB
@@ -1542,10 +1566,10 @@ DEFINE_TESTCASE(crashrecovery1, brass || chert || flint) {
 
 	// Simulate a transaction starting, some of the baseB getting removed,
 	// but then the transaction fails.
-	unlink(path + "/record" + base_ext);
-	unlink(path + "/termlist" + base_ext);
+	unlink((path + "/record" + base_ext).c_str());
+	unlink((path + "/termlist" + base_ext).c_str());
 
-	dbr.reopen();
+	TEST(!dbr.reopen());
 	TEST_EQUAL(dbr.get_doccount(), 2);
     }
 
@@ -1561,7 +1585,7 @@ DEFINE_TESTCASE(crashrecovery1, brass || chert || flint) {
 
     db.add_document(doc);
     db.commit();
-    dbr.reopen();
+    TEST(dbr.reopen());
     TEST_EQUAL(dbr.get_doccount(), 3);
 
     // Xapian::Database has full set of baseB, old baseA
@@ -1574,7 +1598,7 @@ DEFINE_TESTCASE(crashrecovery1, brass || chert || flint) {
 
     db.add_document(doc);
     db.commit();
-    dbr.reopen();
+    TEST(dbr.reopen());
     TEST_EQUAL(dbr.get_doccount(), 4);
 
     return true;
@@ -1592,7 +1616,10 @@ DEFINE_TESTCASE(nomoredocids1, writable) {
     doc.set_data("prose");
     doc.add_term("word");
 
-    db.replace_document(Xapian::docid(-1), doc);
+    // FIXME: This probably should use the _MAX_DOCID values
+    Xapian::docid max_id = 0xffffffff;
+
+    db.replace_document(max_id, doc);
 
     TEST_EXCEPTION(Xapian::DatabaseError, db.add_document(doc));
 
@@ -1750,12 +1777,13 @@ DEFINE_TESTCASE(termtoolong1, writable) {
 
     db.commit();
 
+    size_t limit = endswith(get_dbtype(), "glass") ? 255 : 252;
     {
-	// Currently brass, flint and chert escape zero bytes from terms in
-	// keys for some tables, so a term with 127 zero bytes won't work
-	// either.
+	// Currently chert and glass escape zero bytes from terms in keys for
+	// some tables, so a term with 127 zero bytes won't work for chert, and
+	// with 128 zero bytes won't work for glass.
 	Xapian::Document doc;
-	doc.add_term(string(127, '\0'));
+	doc.add_term(string(limit / 2 + 1, '\0'));
 	db.add_document(doc);
 	try {
 	    db.commit();
@@ -1765,7 +1793,10 @@ DEFINE_TESTCASE(termtoolong1, writable) {
 	    // exception message - we've got this wrong in two different ways
 	    // in the past!
 	    tout << e.get_msg() << endl;
-	    TEST(e.get_msg().find(" is 252 bytes") != string::npos);
+	    string target = " is ";
+	    target += str(limit);
+	    target += " bytes";
+	    TEST(e.get_msg().find(target) != string::npos);
 	}
     }
 
@@ -1789,6 +1820,7 @@ DEFINE_TESTCASE(postlist7, writable) {
     TEST_EQUAL(*p, 5);
     TEST_EQUAL(p.get_wdf(), 3);
     TEST_EQUAL(p.get_doclength(), 7);
+    TEST_EQUAL(p.get_unique_terms(), 2);
     ++p;
     TEST(p == db_w.postlist_end("foo"));
 
@@ -1804,18 +1836,20 @@ DEFINE_TESTCASE(postlist7, writable) {
     TEST_EQUAL(*p, 5);
     TEST_EQUAL(p.get_wdf(), 3);
     TEST_EQUAL(p.get_doclength(), 7);
+    TEST_EQUAL(p.get_unique_terms(), 2);
     ++p;
     TEST(p != db_w.postlist_end("foo"));
     TEST_EQUAL(*p, 6);
     TEST_EQUAL(p.get_wdf(), 1);
     TEST_EQUAL(p.get_doclength(), 2);
+    TEST_EQUAL(p.get_unique_terms(), 2);
     ++p;
     TEST(p == db_w.postlist_end("foo"));
 
     return true;
 }
 
-DEFINE_TESTCASE(lazytablebug1, brass || chert || flint) {
+DEFINE_TESTCASE(lazytablebug1, chert || glass) {
     {
 	Xapian::WritableDatabase db = get_named_writable_database("lazytablebug1", string());
 
@@ -1844,10 +1878,10 @@ DEFINE_TESTCASE(lazytablebug1, brass || chert || flint) {
 
 /** Regression test for bug #287 for flint.
  *
- *  Chert also has the same duff code but this testcase doesn't actually 
+ *  Chert also has the same duff code but this testcase doesn't actually
  *  tickle the bug there.
  */
-DEFINE_TESTCASE(cursordelbug1, brass || chert || flint) {
+DEFINE_TESTCASE(cursordelbug1, chert || glass) {
     static const int terms[] = { 219, 221, 222, 223, 224, 225, 226 };
     static const int copies[] = { 74, 116, 199, 21, 45, 155, 189 };
 
@@ -1873,17 +1907,8 @@ DEFINE_TESTCASE(cursordelbug1, brass || chert || flint) {
 
     db.commit();
 
-    string cmd = XAPIAN_BIN_PATH"xapian-check ";
-    cmd += get_named_writable_database_path("cursordelbug1");
-#ifdef __WIN32__
-    cmd += " >nul";
-#else
-    cmd += " >/dev/null";
-#endif
-    if (system(cmd.c_str()) != 0)
-	return false;
-
-    return true;
+    const string & db_path = get_named_writable_database_path("cursordelbug1");
+    return Xapian::Database::check(db_path) == 0;
 }
 
 /** Helper function for modifyvalues1.
@@ -1893,7 +1918,7 @@ static void
 check_vals(const Xapian::Database & db, const map<Xapian::docid, string> & vals)
 {
     TEST_EQUAL(db.get_doccount(), vals.size());
-    if (vals.size() == 0) return;
+    if (vals.empty()) return;
     TEST_REL(vals.rbegin()->first,<=,db.get_lastdocid());
     map<Xapian::docid, string>::const_iterator i;
     for (i = vals.begin(); i != vals.end(); ++i) {
@@ -1925,13 +1950,13 @@ DEFINE_TESTCASE(modifyvalues1, writable) {
     Xapian::WritableDatabase db = get_writable_database();
     // Note: doccount must be coprime with 13
     const Xapian::doccount doccount = 1000;
-    STATIC_ASSERT(doccount % 13 != 0);
+    static_assert(doccount % 13 != 0, "doccount divisible by 13");
 
     map<Xapian::docid, string> vals;
 
     for (Xapian::doccount num = 1; num <= doccount; ++num) {
 	tout.str(string());
-    	Xapian::Document doc;
+	Xapian::Document doc;
 	string val = "val" + str(num);
 	tout << "Setting val '" << val << "' in doc " << num << "\n";
 	doc.add_value(1, val);
