@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2013 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2012,2013,2015 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,7 +24,9 @@
 
 #include "testsuite.h"
 
-#include "backendmanager.h"
+#ifndef NO_LIBXAPIAN
+# include "backendmanager.h"
+#endif
 #include "fdtracker.h"
 #include "testrunner.h"
 #include "safeunistd.h"
@@ -59,10 +61,12 @@
 # endif
 #endif
 
-#include <xapian/error.h>
+#ifndef NO_LIBXAPIAN
+# include <xapian/error.h>
+#endif
+#include "filetests.h"
 #include "noreturn.h"
 #include "stringutils.h"
-#include "utils.h"
 
 using namespace std;
 
@@ -131,7 +135,7 @@ test_driver::get_srcdir()
     string::size_type i = srcdir.find_last_of(ARGV0_SEP);
     string srcfile;
     if (i != string::npos) {
-	srcfile = srcdir.substr(i + 1);
+	srcfile.assign(srcdir, i + 1, string::npos);
 	srcdir.erase(i);
 	// libtool may put the real executable in .libs.
 	i = srcdir.find_last_of(ARGV0_SEP);
@@ -334,8 +338,10 @@ test_driver::runtest(const test_desc *test)
 		}
 		if (verbose > 1)
 		    write_and_clear_tout();
+#ifndef NO_LIBXAPIAN
 		if (backendmanager)
 		    backendmanager->clean_up();
+#endif
 #ifdef HAVE_VALGRIND
 		if (vg_log_fd != -1) {
 		    // We must empty tout before asking valgrind to perform its
@@ -497,6 +503,7 @@ test_driver::runtest(const test_desc *test)
 		out << col_yellow << " SKIPPED" << col_reset;
 		write_and_clear_tout();
 		return SKIP;
+#ifndef NO_LIBXAPIAN
 	    } catch (const Xapian::Error &err) {
 		string errclass = err.get_type();
 		if (expected_exception && expected_exception == errclass) {
@@ -506,6 +513,7 @@ test_driver::runtest(const test_desc *test)
 		out << " " << col_red << err.get_description() << col_reset;
 		write_and_clear_tout();
 		return FAIL;
+#endif
 	    } catch (const string & msg) {
 		out << col_red << " EXCEPTION std::string " << msg << col_reset;
 		write_and_clear_tout();
@@ -623,8 +631,10 @@ test_driver::do_run_tests(vector<string>::const_iterator b,
 	    out << "Running test: " << test->name << "...";
 	    out.flush();
 	    test_driver::test_result test_res = runtest(test);
+#ifndef NO_LIBXAPIAN
 	    if (backendmanager)
 		backendmanager->clean_up();
+#endif
 	    switch (test_res) {
 		case PASS:
 		    ++res.succeeded;
@@ -786,8 +796,9 @@ test_driver::parse_command_line(int argc, char **argv)
 	if (getenv("XAPIAN_TESTSUITE_VALGRIND") != NULL) {
 	    // Open the valgrind log file, and unlink it.
 	    char fname[64];
-	    sprintf(fname, ".valgrind.log.%lu", (unsigned long)getpid());
-	    vg_log_fd = open(fname, O_RDONLY|O_NONBLOCK);
+	    sprintf(fname, ".valgrind.log.%lu",
+		    static_cast<unsigned long>(getpid()));
+	    vg_log_fd = open(fname, O_RDONLY|O_NONBLOCK|O_CLOEXEC);
 	    if (vg_log_fd != -1) unlink(fname);
 	}
     }
