@@ -1,7 +1,8 @@
-/* testsuite.h: a generic test suite engine
- *
- * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2005,2006,2007,2008,2009,2013,2015 Olly Betts
+/** @file testsuite.h
+ * @brief a generic test suite engine
+ */
+/* Copyright 1999,2000,2001 BrightStation PLC
+ * Copyright 2002,2003,2005,2006,2007,2008,2009,2013,2015,2016 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,7 +25,10 @@
 
 #include "noreturn.h"
 
-#include "output.h"
+#ifndef XAPIAN_UNITTEST
+# include "output.h"
+# define UNITTEST_CHECK_EXCEPTION
+#endif
 
 #include "stringutils.h" // For STRINGIZE().
 
@@ -231,14 +235,16 @@ class test_driver {
 };
 
 /// Display the location at which a testcase occurred, with an explanation.
-#define TESTCASE_LOCN(a) __FILE__":" STRINGIZE(__LINE__) ": " STRINGIZE(a)
+#define TESTCASE_LOCN(a) __FILE__ ":" STRINGIZE(__LINE__) ": " STRINGIZE(a)
 
 /** Test a condition, and display the test with an extra explanation if
  *  the condition fails.
  *  NB: wrapped in do { ... } while (0) so a trailing ';' works correctly.
  */
 #define TEST_AND_EXPLAIN(a, b) do {\
-	if (!(a)) FAIL_TEST(TESTCASE_LOCN(a) << std::endl << b << std::endl);\
+	bool test_and_explain_fail_ = !(a);\
+	UNITTEST_CHECK_EXCEPTION\
+	if (test_and_explain_fail_) FAIL_TEST(TESTCASE_LOCN(a) << std::endl << b << std::endl);\
     } while (0)
 
 /// Test a condition, without an additional explanation for failure.
@@ -246,7 +252,7 @@ class test_driver {
 
 /// Test for equality of two things.
 #define TEST_EQUAL(a, b) TEST_AND_EXPLAIN(((a) == (b)), \
-	"Expected `" STRINGIZE(a) "' and `" STRINGIZE(b) "' to be equal:" \
+	"Expected '" STRINGIZE(a) "' and '" STRINGIZE(b) "' to be equal:" \
 	" were " << (a) << " and " << (b))
 
 /** Test for equality of two strings.
@@ -263,17 +269,17 @@ extern bool TEST_EQUAL_DOUBLE_(double a, double b);
 
 /// Test two doubles for near equality.
 #define TEST_EQUAL_DOUBLE(a, b) TEST_AND_EXPLAIN(TEST_EQUAL_DOUBLE_((a), (b)), \
-	"Expected `" STRINGIZE(a) "' and `" STRINGIZE(b) "' to be (nearly) equal:" \
+	"Expected '" STRINGIZE(a) "' and '" STRINGIZE(b) "' to be (nearly) equal:" \
 	" were " << setprecision(DBL_DIG) << (a) << " and " << (b) << ")" << setprecision(6))
 
 /// Test two doubles for non-near-equality.
 #define TEST_NOT_EQUAL_DOUBLE(a, b) TEST_AND_EXPLAIN(!TEST_EQUAL_DOUBLE_((a), (b)), \
-	"Expected `" STRINGIZE(a) "' and `" STRINGIZE(b) "' not to be (nearly) equal:" \
+	"Expected '" STRINGIZE(a) "' and '" STRINGIZE(b) "' not to be (nearly) equal:" \
 	" were " << setprecision(DBL_DIG) << (a) << " and " << (b) << ")" << setprecision(6))
 
 /// Test for non-equality of two things.
 #define TEST_NOT_EQUAL(a, b) TEST_AND_EXPLAIN(((a) != (b)), \
-	"Expected `" STRINGIZE(a) "' and `" STRINGIZE(b) "' not to be equal:" \
+	"Expected '" STRINGIZE(a) "' and '" STRINGIZE(b) "' not to be equal:" \
 	" were " << (a) << " and " << (b))
 
 #define DEFINE_TESTCASE(S,COND) bool test_##S()
