@@ -2,6 +2,7 @@
  * @brief Xapian::TfIdfWeight class - The TfIdf weighting scheme
  */
 /* Copyright (C) 2013 Aarsh Shah
+ * Copyright (C) 2016 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -47,6 +48,7 @@ TfIdfWeight::TfIdfWeight(const std::string &normals)
     }
     need_stat(WDF);
     need_stat(WDF_MAX);
+    need_stat(WQF);
 }
 
 TfIdfWeight *
@@ -58,7 +60,7 @@ TfIdfWeight::clone() const
 void
 TfIdfWeight::init(double factor_)
 {
-    factor = factor_;
+    factor = get_wqf() * factor_;
 }
 
 string
